@@ -1,9 +1,12 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/GenericInterface/Mapping.pm - GenericInterface data mapping interface
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
+# --
+# $Id: Mapping.pm,v 1.18.2.1 2012-07-30 23:33:18 ep Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::GenericInterface::Mapping;
@@ -13,44 +16,78 @@ use warnings;
 
 use Kernel::System::VariableCheck qw(IsHashRefWithData IsStringWithData);
 
-# prevent 'Used once' warning for Kernel::OM
-use Kernel::System::ObjectManager;
-
-our $ObjectManagerDisabled = 1;
+use vars qw(@ISA $VERSION);
+$VERSION = qw($Revision: 1.18.2.1 $) [1];
 
 =head1 NAME
 
 Kernel::GenericInterface::Mapping - GenericInterface data mapping interface
 
+=head1 SYNOPSIS
+
 =head1 PUBLIC INTERFACE
 
-=head2 new()
+=over 4
+
+=cut
+
+=item new()
 
 create an object.
 
+    use Kernel::Config;
+    use Kernel::System::Encode;
+    use Kernel::System::Log;
+    use Kernel::System::Time;
+    use Kernel::System::Main;
+    use Kernel::System::DB;
     use Kernel::GenericInterface::Debugger;
     use Kernel::GenericInterface::Mapping;
 
+    my $ConfigObject = Kernel::Config->new();
+    my $EncodeObject = Kernel::System::Encode->new(
+        ConfigObject => $ConfigObject,
+    );
+    my $LogObject = Kernel::System::Log->new(
+        ConfigObject => $ConfigObject,
+        EncodeObject => $EncodeObject,
+    );
+    my $TimeObject = Kernel::System::Time->new(
+        ConfigObject => $ConfigObject,
+        LogObject    => $LogObject,
+    );
+    my $MainObject = Kernel::System::Main->new(
+        ConfigObject => $ConfigObject,
+        EncodeObject => $EncodeObject,
+        LogObject    => $LogObject,
+    );
+    my $DBObject = Kernel::System::DB->new(
+        ConfigObject => $ConfigObject,
+        EncodeObject => $EncodeObject,
+        LogObject    => $LogObject,
+        MainObject   => $MainObject,
+    );
     my $DebuggerObject = Kernel::GenericInterface::Debugger->new(
-        DebuggerConfig   => {
-            DebugThreshold  => 'debug',
-            TestMode        => 0,           # optional, in testing mode the data will not be written to the DB
-            # ...
-        },
-        WebserviceID      => 12,
-        CommunicationType => Requester, # Requester or Provider
-        RemoteIP          => 192.168.1.1, # optional
+        ConfigObject       => $ConfigObject,
+        EncodeObject       => $EncodeObject,
+        LogObject          => $LogObject,
+        MainObject         => $MainObject,
+        DBObject           => $DBObject,
+        TimeObject         => $TimeObject,
     );
     my $MappingObject = Kernel::GenericInterface::Mapping->new(
-        DebuggerObject => $DebuggerObject,
-        Invoker        => 'TicketLock',            # the name of the invoker in the web service
-        InvokerType    => 'Nagios::TicketLock',    # the Invoker backend to use
-        Operation      => 'TicketCreate',          # the name of the operation in the web service
-        OperationType  => 'Ticket::TicketCreate',  # the local operation backend to use
-        MappingConfig => {
+        ConfigObject       => $ConfigObject,
+        EncodeObject       => $EncodeObject,
+        LogObject          => $LogObject,
+        MainObject         => $MainObject,
+        DBObject           => $DBObject,
+        TimeObject         => $TimeObject,
+        DebuggerObject     => $DebuggerObject,
+
+        MappingConfig   => {
             Type => 'MappingSimple',
             Config => {
-                # ...
+                ...
             },
         },
     );
@@ -65,9 +102,8 @@ sub new {
     bless( $Self, $Type );
 
     # check needed params
-    for my $Needed (qw(DebuggerObject MappingConfig)) {
+    for my $Needed (qw(ConfigObject DBObject DebuggerObject MainObject MappingConfig)) {
         if ( !$Param{$Needed} ) {
-
             return {
                 Success      => 0,
                 ErrorMessage => "Got no $Needed!"
@@ -77,23 +113,13 @@ sub new {
         $Self->{$Needed} = $Param{$Needed};
     }
 
-    # add optional params
-    OPTIONAL:
-    for my $Optional (qw(Invoker InvokerType Operation OperationType)) {
-        next OPTIONAL if !$Param{$Optional};
-
-        $Self->{$Optional} = $Param{$Optional};
-    }
-
     # check config - we need at least a config type
     if ( !IsHashRefWithData( $Param{MappingConfig} ) ) {
-
         return $Self->{DebuggerObject}->Error(
             Summary => 'Got no MappingConfig as hash ref with content!',
         );
     }
     if ( !IsStringWithData( $Param{MappingConfig}->{Type} ) ) {
-
         return $Self->{DebuggerObject}->Error(
             Summary => 'Got no MappingConfig with Type as string with value!',
         );
@@ -105,7 +131,6 @@ sub new {
         && !IsHashRefWithData( $Param{MappingConfig}->{Config} )
         )
     {
-
         return $Self->{DebuggerObject}->Error(
             Summary => 'Got MappingConfig with Data, but Data is no hash ref with content!',
         );
@@ -113,8 +138,7 @@ sub new {
 
     # load backend module
     my $GenericModule = 'Kernel::GenericInterface::Mapping::' . $Param{MappingConfig}->{Type};
-    if ( !$Kernel::OM->Get('Kernel::System::Main')->Require($GenericModule) ) {
-
+    if ( !$Self->{MainObject}->Require($GenericModule) ) {
         return $Self->{DebuggerObject}->Error( Summary => "Can't load mapping backend module!" );
     }
     $Self->{BackendObject} = $GenericModule->new( %{$Self} );
@@ -125,7 +149,7 @@ sub new {
     return $Self;
 }
 
-=head2 Map()
+=item Map()
 
 perform data mapping in backend
 
@@ -150,15 +174,12 @@ sub Map {
 
     # check data - only accept undef or hash ref
     if ( defined $Param{Data} && ref $Param{Data} ne 'HASH' ) {
-
-        return $Self->{DebuggerObject}->Error(
-            Summary => 'Got Data but it is not a hash ref in Mapping handler!'
-        );
+        return $Self->{DebuggerObject}
+            ->Error( Summary => 'Got Data but it is not a hash ref in Mapping handler!' );
     }
 
     # return if data is empty
     if ( !defined $Param{Data} || !%{ $Param{Data} } ) {
-
         return {
             Success => 1,
             Data    => {},
@@ -171,12 +192,20 @@ sub Map {
 
 1;
 
+=back
+
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
+
+=cut
+
+=head1 VERSION
+
+$Revision: 1.18.2.1 $ $Date: 2012-07-30 23:33:18 $
 
 =cut
