@@ -1,9 +1,12 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/GenericInterface/Operation/Ticket/TicketGet.pm - GenericInterface Ticket Get operation backend
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
+# --
+# $Id: TicketGet.pm,v 1.12.2.1 2012-11-09 23:17:30 cr Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::GenericInterface::Operation::Ticket::TicketGet;
@@ -12,23 +15,27 @@ use strict;
 use warnings;
 
 use MIME::Base64;
-
+use Kernel::System::Ticket;
+use Kernel::GenericInterface::Operation::Common;
+use Kernel::GenericInterface::Operation::Ticket::Common;
 use Kernel::System::VariableCheck qw(IsArrayRefWithData IsHashRefWithData IsStringWithData);
 
-use parent qw(
-    Kernel::GenericInterface::Operation::Common
-    Kernel::GenericInterface::Operation::Ticket::Common
-);
-
-our $ObjectManagerDisabled = 1;
+use vars qw(@ISA $VERSION);
+$VERSION = qw($Revision: 1.12.2.1 $) [1];
 
 =head1 NAME
 
 Kernel::GenericInterface::Operation::Ticket::TicketGet - GenericInterface Ticket Get Operation backend
 
+=head1 SYNOPSIS
+
 =head1 PUBLIC INTERFACE
 
-=head2 new()
+=over 4
+
+=cut
+
+=item new()
 
 usually, you want to create an instance of this
 by using Kernel::GenericInterface::Operation->new();
@@ -42,48 +49,54 @@ sub new {
     bless( $Self, $Type );
 
     # check needed objects
-    for my $Needed (qw(DebuggerObject WebserviceID)) {
+    for my $Needed (
+        qw(DebuggerObject ConfigObject MainObject LogObject TimeObject DBObject EncodeObject WebserviceID)
+        )
+    {
         if ( !$Param{$Needed} ) {
             return {
                 Success      => 0,
-                ErrorMessage => "Got no $Needed!",
+                ErrorMessage => "Got no $Needed!"
             };
         }
 
         $Self->{$Needed} = $Param{$Needed};
     }
 
+    # create additional objects
+    $Self->{CommonObject} = Kernel::GenericInterface::Operation::Common->new( %{$Self} );
+    $Self->{TicketCommonObject}
+        = Kernel::GenericInterface::Operation::Ticket::Common->new( %{$Self} );
+    $Self->{TicketObject} = Kernel::System::Ticket->new( %{$Self} );
+
     return $Self;
 }
 
-=head2 Run()
+=item Run()
 
 perform TicketGet Operation. This function is able to return
 one or more ticket entries in one call.
 
     my $Result = $OperationObject->Run(
         Data => {
-            UserLogin            => 'some agent login',                            # UserLogin or CustomerUserLogin or SessionID is
-                                                                                   #   required
-            CustomerUserLogin    => 'some customer login',
-            SessionID            => 123,
+            UserLogin         => 'some agent login',                            # UserLogin or CustomerUserLogin or SessionID is
+                                                                                #   required
+            CustomerUserLogin => 'some customer login',
+            SessionID         => 123,
 
-            Password             => 'some password',                               # if UserLogin or customerUserLogin is sent then
-                                                                                   #   Password is required
-            TicketID             => '32,33',                                       # required, could be coma separated IDs or an Array
-            DynamicFields        => 0,                                             # Optional, 0 as default. Indicate if Dynamic Fields
-                                                                                   #     should be included or not on the ticket content.
-            Extended             => 1,                                             # Optional, 0 as default
-            AllArticles          => 1,                                             # Optional, 0 as default. Set as 1 will include articles
-                                                                                   #     for tickets.
-            ArticleSenderType    => [ $ArticleSenderType1, $ArticleSenderType2 ],  # Optional, only requested article sender types
-            ArticleOrder         => 'DESC',                                        # Optional, DESC,ASC - default is ASC
-            ArticleLimit         => 5,                                             # Optional
-            Attachments          => 1,                                             # Optional, 0 as default. If it's set with the value 1,
-                                                                                   # attachments for articles will be included on ticket data
-            GetAttachmentContents = 1                                              # Optional, 1 as default. 0|1,
-            HTMLBodyAsAttachment => 1                                              # Optional, If enabled the HTML body version of each article
-                                                                                   #    is added to the attachments list
+            Password          => 'some password',                                       # if UserLogin or customerUserLogin is sent then
+                                                                                #   Password is required
+            TicketID          => '32,33',                                       # required, could be coma separated IDs or an Array
+            DynamicFields     => 0,                                             # Optional, 0 as default. Indicate if Dynamic Fields
+                                                                                # should be included or not on the ticket content.
+            Extended          => 1,                                             # Optional, 0 as default
+            AllArticles       => 1,                                             # Optional, 0 as default. Set as 1 will include articles
+                                                                                # for tickets.
+            ArticleSenderType => [ $ArticleSenderType1, $ArticleSenderType2 ],  # Optional, only requested article sender types
+            ArticleOrder      => 'DESC',                                        # Optional, DESC,ASC - default is ASC
+            ArticleLimit      => 5,                                             # Optional
+            Attachments       => 1,                                             # Optional, 1 as default. If it's set with the value 1,
+                                                                                # attachments for articles will be included on ticket data
         },
     );
 
@@ -119,19 +132,14 @@ one or more ticket entries in one call.
                     ResponsibleID      => 123,
                     Age                => 3456,
                     Created            => '2010-10-27 20:15:00'
+                    CreateTimeUnix     => '1231414141',
                     CreateBy           => 123,
                     Changed            => '2010-10-27 20:15:15',
                     ChangeBy           => 123,
                     ArchiveFlag        => 'y',
-                    TimeUnit           => 123,
 
                     # If DynamicFields => 1 was passed, you'll get an entry like this for each dynamic field:
-                    DynamicField => [
-                        {
-                            Name  => 'some name',
-                            Value => 'some value',
-                        },
-                    ],
+                    DynamicField_X     => 'value_x',
 
                     # (time stamps of expected escalations)
                     EscalationResponseTime           (unix time stamp of response time escalation)
@@ -172,6 +180,7 @@ one or more ticket entries in one call.
                     FirstResponseInMin              (minutes till first response)
                     FirstResponseDiffInMin          (minutes till or over first response)
 
+                    SolutionTime                    (timestamp of solution time, also close time)
                     SolutionInMin                   (minutes till solution time)
                     SolutionDiffInMin               (minutes till or over solution time)
 
@@ -191,20 +200,15 @@ one or more ticket entries in one call.
                             References
                             SenderType
                             SenderTypeID
-                            IsVisibleForCustomer
+                            ArticleType
+                            ArticleTypeID
                             ContentType
                             Charset
                             MimeType
                             IncomingTime
-                            TimeUnit
 
                             # If DynamicFields => 1 was passed, you'll get an entry like this for each dynamic field:
-                            DynamicField => [
-                                {
-                                    Name  => 'some name',
-                                    Value => 'some value',
-                                },
-                            ],
+                            DynamicField_X     => 'value_x',
 
                             Attachment => [
                                 {
@@ -212,8 +216,8 @@ one or more ticket entries in one call.
                                     ContentAlternative => "",
                                     ContentID          => "",
                                     ContentType        => "application/pdf",
-                                    FileID             => 34,
                                     Filename           => "StdAttachment-Test1.pdf",
+                                    Filesize           => "4.6 KBytes",
                                     FilesizeRaw        => 4722,
                                 },
                                 {
@@ -238,22 +242,11 @@ one or more ticket entries in one call.
 sub Run {
     my ( $Self, %Param ) = @_;
 
-    my $Result = $Self->Init(
-        WebserviceID => $Self->{WebserviceID},
+    my ( $UserID, $UserType ) = $Self->{CommonObject}->Auth(
+        %Param
     );
 
-    if ( !$Result->{Success} ) {
-        return $Self->ReturnError(
-            ErrorCode    => 'Webservice.InvalidConfiguration',
-            ErrorMessage => $Result->{ErrorMessage},
-        );
-    }
-
-    my ( $UserID, $UserType ) = $Self->Auth(
-        %Param,
-    );
-
-    return $Self->ReturnError(
+    return $Self->{TicketCommonObject}->ReturnError(
         ErrorCode    => 'TicketGet.AuthFail',
         ErrorMessage => "TicketGet: Authorization failing!",
     ) if !$UserID;
@@ -261,7 +254,7 @@ sub Run {
     # check needed stuff
     for my $Needed (qw(TicketID)) {
         if ( !$Param{Data}->{$Needed} ) {
-            return $Self->ReturnError(
+            return $Self->{TicketCommonObject}->ReturnError(
                 ErrorCode    => 'TicketGet.MissingParameter',
                 ErrorMessage => "TicketGet: $Needed parameter is missing!",
             );
@@ -269,7 +262,7 @@ sub Run {
     }
     my $ErrorMessage = '';
 
-    # all needed variables
+    # all needed vairables
     my @TicketIDs;
     if ( IsStringWithData( $Param{Data}->{TicketID} ) ) {
         @TicketIDs = split( /,/, $Param{Data}->{TicketID} );
@@ -278,117 +271,46 @@ sub Run {
         @TicketIDs = @{ $Param{Data}->{TicketID} };
     }
     else {
-        return $Self->ReturnError(
+        return $Self->{TicketCommonObject}->ReturnError(
             ErrorCode    => 'TicketGet.WrongStructure',
             ErrorMessage => "TicketGet: Structure for TicketID is not correct!",
         );
     }
-
-    # Get the list of article dynamic fields
-    my $ArticleDynamicFieldList = $Kernel::OM->Get('Kernel::System::DynamicField')->DynamicFieldList(
-        ObjectType => 'Article',
-        ResultType => 'HASH',
-    );
-
-    # Crate a lookup list for easy search
-    my %ArticleDynamicFieldLookup = reverse %{$ArticleDynamicFieldList};
-
-    TICKET:
-    for my $TicketID (@TicketIDs) {
-
-        my $Access = $Self->CheckAccessPermissions(
-            TicketID => $TicketID,
-            UserID   => $UserID,
-            UserType => $UserType,
-        );
-
-        next TICKET if $Access;
-
-        return $Self->ReturnError(
-            ErrorCode    => 'TicketGet.AccessDenied',
-            ErrorMessage => 'TicketGet: User does not have access to the ticket!',
-        );
-    }
-
-    my $DynamicFields = $Param{Data}->{DynamicFields} || 0;
-    my $Extended      = $Param{Data}->{Extended}      || 0;
-    my $AllArticles   = $Param{Data}->{AllArticles}   || 0;
-    my $ArticleOrder  = $Param{Data}->{ArticleOrder}  || 'ASC';
-    my $ArticleLimit  = $Param{Data}->{ArticleLimit}  || 0;
-    my $Attachments   = $Param{Data}->{Attachments}   || 0;
-    my $GetAttachmentContents = $Param{Data}->{GetAttachmentContents} // 1;
-
-    my $ReturnData = {
+    my $DynamicFields     = $Param{Data}->{DynamicFields}     || 0;
+    my $Extended          = $Param{Data}->{Extended}          || 0;
+    my $AllArticles       = $Param{Data}->{AllArticles}       || 0;
+    my $ArticleSenderType = $Param{Data}->{ArticleSenderType} || '';
+    my $ArticleOrder      = $Param{Data}->{ArticleOrder}      || 'ASC';
+    my $ArticleLimit      = $Param{Data}->{ArticleLimit}      || 0;
+    my $Attachments       = $Param{Data}->{Attachments}       || 0;
+    my $ReturnData        = {
         Success => 1,
     };
     my @Item;
 
-    my $ArticleSenderType = '';
-    if ( IsArrayRefWithData( $Param{Data}->{ArticleSenderType} ) ) {
-        $ArticleSenderType = $Param{Data}->{ArticleSenderType};
-    }
-    elsif ( IsStringWithData( $Param{Data}->{ArticleSenderType} ) ) {
-        $ArticleSenderType = [ $Param{Data}->{ArticleSenderType} ];
-    }
-
-    # By default, do not include HTML body as attachment, unless it is explicitly requested.
-    my %ExcludeAttachments = (
-        ExcludePlainText => 1,
-        ExcludeHTMLBody  => $Param{Data}->{HTMLBodyAsAttachment} ? 0 : 1,
-    );
-
     # start ticket loop
     TICKET:
     for my $TicketID (@TicketIDs) {
 
-        my $TicketObject = $Kernel::OM->Get('Kernel::System::Ticket');
-
         # get the Ticket entry
-        my %TicketEntryRaw = $TicketObject->TicketGet(
+        my %TicketEntry = $Self->{TicketObject}->TicketGet(
             TicketID      => $TicketID,
             DynamicFields => $DynamicFields,
             Extended      => $Extended,
             UserID        => $UserID,
         );
 
-        if ( !IsHashRefWithData( \%TicketEntryRaw ) ) {
+        if ( !IsHashRefWithData( \%TicketEntry ) ) {
 
             $ErrorMessage = 'Could not get Ticket data'
                 . ' in Kernel::GenericInterface::Operation::Ticket::TicketGet::Run()';
 
-            return $Self->ReturnError(
+            return $Self->{TicketCommonObject}->ReturnError(
                 ErrorCode    => 'TicketGet.NotValidTicketID',
                 ErrorMessage => "TicketGet: $ErrorMessage",
             );
         }
 
-        my %TicketEntry;
-        my @DynamicFields;
-
-        # remove all dynamic fields from main ticket hash and set them into an array.
-        ATTRIBUTE:
-        for my $Attribute ( sort keys %TicketEntryRaw ) {
-
-            if ( $Attribute =~ m{\A DynamicField_(.*) \z}msx ) {
-                push @DynamicFields, {
-                    Name  => $1,
-                    Value => $TicketEntryRaw{$Attribute},
-                };
-                next ATTRIBUTE;
-            }
-
-            $TicketEntry{$Attribute} = $TicketEntryRaw{$Attribute};
-        }
-
-        $TicketEntry{TimeUnit} = $TicketObject->TicketAccountedTimeGet(
-            TicketID => $TicketID,
-        );
-
-        # add dynamic fields array into 'DynamicField' hash key if any
-        if (@DynamicFields) {
-            $TicketEntry{DynamicField} = \@DynamicFields;
-        }
-
         # set Ticket entry data
         my $TicketBundle = {
             %TicketEntry,
@@ -399,91 +321,50 @@ sub Run {
             next TICKET;
         }
 
-        my %ArticleListFilters;
-        if ( $UserType eq 'Customer' ) {
-            %ArticleListFilters = (
-                IsVisibleForCustomer => 1,
-            );
-        }
-
-        my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
-
-        my @Articles;
-        if ($ArticleSenderType) {
-            for my $SenderType ( @{ $ArticleSenderType || [] } ) {
-                my @ArticlesFiltered = $ArticleObject->ArticleList(
-                    TicketID   => $TicketID,
-                    SenderType => $SenderType,
-                    %ArticleListFilters,
-                );
-                push @Articles, @ArticlesFiltered;
-            }
-        }
-        else {
-            @Articles = $ArticleObject->ArticleList(
-                TicketID => $TicketID,
-                %ArticleListFilters,
-            );
-        }
-
-        # Modify ArticleLimit if it is greater then number of articles (see bug#14585).
-        if ( $ArticleLimit > scalar @Articles ) {
-            $ArticleLimit = scalar @Articles;
-        }
-
-        # Set number of articles by ArticleLimit and ArticleOrder parameters.
-        if ( IsArrayRefWithData( \@Articles ) && $ArticleLimit ) {
-            if ( $ArticleOrder eq 'DESC' ) {
-                @Articles = reverse @Articles;
-            }
-            @Articles = @Articles[ 0 .. ( $ArticleLimit - 1 ) ];
-        }
+        my @ArticleBox = $Self->{TicketObject}->ArticleGet(
+            TicketID          => $TicketID,
+            ArticleSenderType => $ArticleSenderType,
+            DynamicFields     => $DynamicFields,
+            Extended          => $Extended,
+            Order             => $ArticleOrder,
+            Limit             => $ArticleLimit,
+            UserID            => $UserID,
+        );
 
         # start article loop
         ARTICLE:
-        for my $Article (@Articles) {
-
-            my $ArticleBackendObject = $ArticleObject->BackendForArticle( %{$Article} );
-
-            my %ArticleData = $ArticleBackendObject->ArticleGet(
-                TicketID      => $TicketID,
-                ArticleID     => $Article->{ArticleID},
-                DynamicFields => $DynamicFields,
-            );
-            $Article = \%ArticleData;
+        for my $Article (@ArticleBox) {
 
+            # next if not attachments required
             next ARTICLE if !$Attachments;
 
             # get attachment index (without attachments)
-            my %AtmIndex = $ArticleBackendObject->ArticleAttachmentIndex(
-                ArticleID => $Article->{ArticleID},
-                %ExcludeAttachments,
+            my %AtmIndex = $Self->{TicketObject}->ArticleAttachmentIndex(
+                ContentPath                => $Article->{ContentPath},
+                ArticleID                  => $Article->{ArticleID},
+                StripPlainBodyAsAttachment => 3,
+                Article                    => $Article,
+                UserID                     => $UserID,
             );
 
+            # next if not attachments
             next ARTICLE if !IsHashRefWithData( \%AtmIndex );
 
             my @Attachments;
             ATTACHMENT:
-            for my $FileID ( sort keys %AtmIndex ) {
+            for my $FileID ( keys %AtmIndex ) {
                 next ATTACHMENT if !$FileID;
-                my %Attachment = $ArticleBackendObject->ArticleAttachment(
+                my %Attachment = $Self->{TicketObject}->ArticleAttachment(
                     ArticleID => $Article->{ArticleID},
                     FileID    => $FileID,                 # as returned by ArticleAttachmentIndex
+                    UserID    => $UserID,
                 );
 
+                # next if not attachment
                 next ATTACHMENT if !IsHashRefWithData( \%Attachment );
 
-                $Attachment{FileID} = $FileID;
-                if ($GetAttachmentContents)
-                {
-                    # convert content to base64, but prevent 76 chars brake, see bug#14500.
-                    $Attachment{Content} = encode_base64( $Attachment{Content}, '' );
-                }
-                else {
-                    # unset content
-                    $Attachment{Content}            = '';
-                    $Attachment{ContentAlternative} = '';
-                }
+                # convert content to base64
+                $Attachment{Content} = encode_base64( $Attachment{Content} );
                 push @Attachments, {%Attachment};
             }
 
@@ -493,47 +374,7 @@ sub Run {
         }    # finish article loop
 
         # set Ticket entry data
-        if (@Articles) {
-
-            my @ArticleBox;
-
-            for my $ArticleRaw (@Articles) {
-                my %Article;
-                my @ArticleDynamicFields;
-
-                # remove all dynamic fields from main article hash and set them into an array.
-                ATTRIBUTE:
-                for my $Attribute ( sort keys %{$ArticleRaw} ) {
-
-                    if ( $Attribute =~ m{\A DynamicField_(.*) \z}msx ) {
-
-                        # skip dynamic fields that are not article related
-                        # this is needed because ArticleGet() also returns ticket dynamic fields
-                        next ATTRIBUTE if ( !$ArticleDynamicFieldLookup{$1} );
-
-                        push @ArticleDynamicFields, {
-                            Name  => $1,
-                            Value => $ArticleRaw->{$Attribute},
-                        };
-                        next ATTRIBUTE;
-                    }
-
-                    $Article{$Attribute} = $ArticleRaw->{$Attribute};
-                }
-
-                $Article{TimeUnit} = $ArticleObject->ArticleAccountedTimeGet(
-                    ArticleID => $ArticleRaw->{ArticleID}
-                );
-
-                # add dynamic fields array into 'DynamicField' hash key if any
-                if (@ArticleDynamicFields) {
-                    $Article{DynamicField} = \@ArticleDynamicFields;
-                }
-
-                push @ArticleBox, \%Article;
-            }
-            $TicketBundle->{Article} = \@ArticleBox;
-        }
+        $TicketBundle->{Article} = \@ArticleBox;
 
         # add
         push @Item, $TicketBundle;
@@ -543,7 +384,7 @@ sub Run {
         $ErrorMessage = 'Could not get Ticket data'
             . ' in Kernel::GenericInterface::Operation::Ticket::TicketGet::Run()';
 
-        return $Self->ReturnError(
+        return $Self->{TicketCommonObject}->ReturnError(
             ErrorCode    => 'TicketGet.NotTicketData',
             ErrorMessage => "TicketGet: $ErrorMessage",
         );
@@ -559,12 +400,20 @@ sub Run {
 
 1;
 
+=back
+
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
+
+=cut
+
+=head1 VERSION
+
+$Revision: 1.12.2.1 $ $Date: 2012-11-09 23:17:30 $
 
 =cut
