@@ -1,6 +1,6 @@
 # --
 # Kernel/GenericInterface/Transport/HTTP/SOAP.pm - GenericInterface network transport interface for HTTP::SOAP
-# Copyright (C) 2001-2012 OTRS AG, http://otrs.org/
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
 # $Id: SOAP.pm,v 1.45.2.5 2012-12-05 15:06:30 cr Exp $
 # --
@@ -344,7 +344,7 @@ sub ProviderGenerateResponse {
     }
     my $Serialized = SOAP::Serializer
         ->autotype(0)
-        ->ns( $Self->{TransportConfig}->{Config}->{NameSpace} )
+        ->default_ns( $Self->{TransportConfig}->{Config}->{NameSpace} )
         ->envelope(@CallData);
     my $SerializedFault = $@ || '';
     if ($SerializedFault) {
@@ -808,7 +808,7 @@ sub _Output {
 
     # set keep-alive
     my $ConfigKeepAlive = $Self->{ConfigObject}->Get('SOAP::Keep-Alive');
-    my $Connection      = $ConfigKeepAlive ? 'Keep-Alive' : 'close';
+    my $Connection = $ConfigKeepAlive ? 'Keep-Alive' : 'close';
 
     # in the constructor of this module STDIN and STDOUT are set to binmode without any additional
     # layer (according to the documentation this is the same as set :raw). Previous solutions for
