@@ -1,9 +1,12 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/Modules/AdminDynamicFieldMultiselect.pm - provides a dynamic fields text config view for admins
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
+# --
+# $Id: AdminDynamicFieldMultiselect.pm,v 1.8 2012-04-18 19:38:01 cr Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::Modules::AdminDynamicFieldMultiselect;
@@ -11,10 +14,13 @@ package Kernel::Modules::AdminDynamicFieldMultiselect;
 use strict;
 use warnings;
 
-our $ObjectManagerDisabled = 1;
-
 use Kernel::System::VariableCheck qw(:all);
-use Kernel::Language qw(Translatable);
+use Kernel::System::Valid;
+use Kernel::System::CheckItem;
+use Kernel::System::DynamicField;
+
+use vars qw($VERSION);
+$VERSION = qw($Revision: 1.8 $) [1];
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -22,6 +28,23 @@ sub new {
     my $Self = {%Param};
     bless( $Self, $Type );
 
+    for (qw(ParamObject LayoutObject LogObject ConfigObject)) {
+        if ( !$Self->{$_} ) {
+            $Self->{LayoutObject}->FatalError( Message => "Got no $_!" );
+        }
+    }
+
+    # create additional objects
+    $Self->{ValidObject} = Kernel::System::Valid->new( %{$Self} );
+
+    $Self->{DynamicFieldObject} = Kernel::System::DynamicField->new( %{$Self} );
+
+    # get configured object types
+    $Self->{ObjectTypeConfig} = $Self->{ConfigObject}->Get('DynamicFields::ObjectType');
+
+    # get the fields config
+    $Self->{FieldTypeConfig} = $Self->{ConfigObject}->Get('DynamicFields::Backend') || {};
+
     # set possible values handling strings
     $Self->{EmptyString}     = '_DynamicFields_EmptyString_Dont_Use_It_String_Please';
     $Self->{DuplicateString} = '_DynamicFields_DuplicatedString_Dont_Use_It_String_Please';
@@ -33,15 +56,6 @@ sub new {
 sub Run {
     my ( $Self, %Param ) = @_;
 
-    # Store last entity screen.
-    $Kernel::OM->Get('Kernel::System::AuthSession')->UpdateSessionID(
-        SessionID => $Self->{SessionID},
-        Key       => 'LastScreenEntity',
-        Value     => $Self->{RequestedURL},
-    );
-
-    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
-
     if ( $Self->{Subaction} eq 'Add' ) {
         return $Self->_Add(
             %Param,
@@ -50,7 +64,7 @@ sub Run {
     elsif ( $Self->{Subaction} eq 'AddAction' ) {
 
         # challenge token check for write action
-        $LayoutObject->ChallengeTokenCheck();
+        $Self->{LayoutObject}->ChallengeTokenCheck();
 
         return $Self->_AddAction(
             %Param,
@@ -64,44 +78,38 @@ sub Run {
     elsif ( $Self->{Subaction} eq 'ChangeAction' ) {
 
         # challenge token check for write action
-        $LayoutObject->ChallengeTokenCheck();
+        $Self->{LayoutObject}->ChallengeTokenCheck();
 
         return $Self->_ChangeAction(
             %Param,
         );
     }
-    return $LayoutObject->ErrorScreen(
-        Message => Translatable('Undefined subaction.'),
+    return $Self->{LayoutObject}->ErrorScreen(
+        Message => "Undefined subaction.",
     );
 }
 
 sub _Add {
     my ( $Self, %Param ) = @_;
 
-    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
-
     my %GetParam;
     for my $Needed (qw(ObjectType FieldType FieldOrder)) {
-        $GetParam{$Needed} = $Kernel::OM->Get('Kernel::System::Web::Request')->GetParam( Param => $Needed );
-        if ( !$GetParam{$Needed} ) {
-            return $LayoutObject->ErrorScreen(
-                Message => $LayoutObject->{LanguageObject}->Translate( 'Need %s', $Needed ),
+        $GetParam{$Needed} = $Self->{ParamObject}->GetParam( Param => $Needed );
+        if ( !$Needed ) {
+            return $Self->{LayoutObject}->ErrorScreen(
+                Message => "Need $Needed",
             );
         }
     }
 
     # get the object type and field type display name
-    my $ConfigObject   = $Kernel::OM->Get('Kernel::Config');
-    my $ObjectTypeName = $ConfigObject->Get('DynamicFields::ObjectType')->{ $GetParam{ObjectType} }->{DisplayName}
-        || '';
-    my $FieldTypeName = $ConfigObject->Get('DynamicFields::Driver')->{ $GetParam{FieldType} }->{DisplayName} || '';
+    my $ObjectTypeName = $Self->{ObjectTypeConfig}->{ $GetParam{ObjectType} }->{DisplayName} || '';
+    my $FieldTypeName  = $Self->{FieldTypeConfig}->{ $GetParam{FieldType} }->{DisplayName}   || '';
 
     return $Self->_ShowScreen(
         %Param,
         %GetParam,
         Mode           => 'Add',
-        BreadcrumbText => $LayoutObject->{LanguageObject}
-            ->Translate( 'Add %s field', $LayoutObject->{LanguageObject}->Translate($FieldTypeName) ),
         ObjectTypeName => $ObjectTypeName,
         FieldTypeName  => $FieldTypeName,
     );
@@ -112,83 +120,69 @@ sub _AddAction {
 
     my %Errors;
     my %GetParam;
-    my $ParamObject = $Kernel::OM->Get('Kernel::System::Web::Request');
 
     for my $Needed (qw(Name Label FieldOrder)) {
-        $GetParam{$Needed} = $ParamObject->GetParam( Param => $Needed );
+        $GetParam{$Needed} = $Self->{ParamObject}->GetParam( Param => $Needed );
         if ( !$GetParam{$Needed} ) {
             $Errors{ $Needed . 'ServerError' }        = 'ServerError';
-            $Errors{ $Needed . 'ServerErrorMessage' } = Translatable('This field is required.');
+            $Errors{ $Needed . 'ServerErrorMessage' } = 'This field is required.';
         }
     }
 
-    # get the TreeView option and set it to '0' if it is undefined
-    $GetParam{TreeView} = $ParamObject->GetParam( Param => 'TreeView' );
-    $GetParam{TreeView} = defined $GetParam{TreeView} && $GetParam{TreeView} ? '1' : '0';
-
-    my $DynamicFieldObject = $Kernel::OM->Get('Kernel::System::DynamicField');
-
     if ( $GetParam{Name} ) {
 
         # check if name is alphanumeric
-        if ( $GetParam{Name} !~ m{\A (?: [a-zA-Z] | \d )+ \z}xms ) {
+        if ( $GetParam{Name} !~ m{\A ( ?: [a-zA-Z] | \d )+ \z}xms ) {
 
             # add server error error class
             $Errors{NameServerError} = 'ServerError';
             $Errors{NameServerErrorMessage} =
-                Translatable('The field does not contain only ASCII letters and numbers.');
+                'The field does not contain only ASCII letters and numbers.';
         }
 
-        # get dynamic field list
-        my $DynamicFieldsList = $DynamicFieldObject->DynamicFieldList(
-            Valid      => 0,
-            ResultType => 'HASH',
-        ) || {};
-
         # check if name is duplicated
-        my %DynamicFieldsList = %{$DynamicFieldsList};
+        my %DynamicFieldsList = %{
+            $Self->{DynamicFieldObject}->DynamicFieldList(
+                Valid      => 0,
+                ResultType => 'HASH',
+                )
+        };
+
         %DynamicFieldsList = reverse %DynamicFieldsList;
 
         if ( $DynamicFieldsList{ $GetParam{Name} } ) {
 
             # add server error error class
             $Errors{NameServerError}        = 'ServerError';
-            $Errors{NameServerErrorMessage} = Translatable('There is another field with the same name.');
+            $Errors{NameServerErrorMessage} = 'There is another field with the same name.';
         }
     }
 
     if ( $GetParam{FieldOrder} ) {
 
         # check if field order is numeric and positive
-        if ( $GetParam{FieldOrder} !~ m{\A (?: \d )+ \z}xms ) {
+        if ( $GetParam{FieldOrder} !~ m{\A ( ?: \d )+ \z}xms ) {
 
             # add server error error class
             $Errors{FieldOrderServerError}        = 'ServerError';
-            $Errors{FieldOrderServerErrorMessage} = Translatable('The field must be numeric.');
+            $Errors{FieldOrderServerErrorMessage} = 'The field must be numeric.';
         }
     }
 
     for my $ConfigParam (
         qw(
-        ObjectType ObjectTypeName FieldType FieldTypeName PossibleNone
+        ObjectType ObjectTypeName FieldType FieldTypeName DefaultValue PossibleNone
         TranslatableValues ValidID
         )
         )
     {
-        $GetParam{$ConfigParam} = $ParamObject->GetParam( Param => $ConfigParam );
+        $GetParam{$ConfigParam} = $Self->{ParamObject}->GetParam( Param => $ConfigParam );
     }
 
-    # get default values
-    my @DefaultValues = $ParamObject->GetArray( Param => 'DefaultValue' );
-    $GetParam{DefaultValue} = \@DefaultValues;
-
-    # get layout object
-    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
-
     # uncorrectable errors
     if ( !$GetParam{ValidID} ) {
-        return $LayoutObject->ErrorScreen(
-            Message => Translatable('Need ValidID'),
+        return $Self->{LayoutObject}->ErrorScreen(
+            Message => "Need ValidID",
         );
     }
 
@@ -196,7 +190,7 @@ sub _AddAction {
 
     # set errors for possible values entries
     KEY:
-    for my $Key ( sort keys %{$PossibleValues} ) {
+    for my $Key ( keys %{$PossibleValues} ) {
 
         # check for empty original values
         if ( $Key =~ m{\A $Self->{EmptyString} (?: \d+)}smx ) {
@@ -234,14 +228,13 @@ sub _AddAction {
     # set specific config
     my $FieldConfig = {
         PossibleValues     => $PossibleValues,
-        TreeView           => $GetParam{TreeView},
         DefaultValue       => $GetParam{DefaultValue},
         PossibleNone       => $GetParam{PossibleNone},
         TranslatableValues => $GetParam{TranslatableValues},
     };
 
     # create a new field
-    my $FieldID = $DynamicFieldObject->DynamicFieldAdd(
+    my $FieldID = $Self->{DynamicFieldObject}->DynamicFieldAdd(
         Name       => $GetParam{Name},
         Label      => $GetParam{Label},
         FieldOrder => $GetParam{FieldOrder},
@@ -253,12 +246,12 @@ sub _AddAction {
     );
 
     if ( !$FieldID ) {
-        return $LayoutObject->ErrorScreen(
-            Message => Translatable('Could not create the new field'),
+        return $Self->{LayoutObject}->ErrorScreen(
+            Message => "Could not create the new field",
         );
     }
 
-    return $LayoutObject->Redirect(
+    return $Self->{LayoutObject}->Redirect(
         OP => "Action=AdminDynamicField",
     );
 }
@@ -266,43 +259,37 @@ sub _AddAction {
 sub _Change {
     my ( $Self, %Param ) = @_;
 
-    my $ParamObject  = $Kernel::OM->Get('Kernel::System::Web::Request');
-    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
     my %GetParam;
-
     for my $Needed (qw(ObjectType FieldType)) {
-        $GetParam{$Needed} = $ParamObject->GetParam( Param => $Needed );
-        if ( !$GetParam{$Needed} ) {
-            return $LayoutObject->ErrorScreen(
-                Message => $LayoutObject->{LanguageObject}->Translate( 'Need %s', $Needed ),
+        $GetParam{$Needed} = $Self->{ParamObject}->GetParam( Param => $Needed );
+        if ( !$Needed ) {
+            return $Self->{LayoutObject}->ErrorScreen(
+                Message => "Need $Needed",
             );
         }
     }
 
     # get the object type and field type display name
-    my $ConfigObject   = $Kernel::OM->Get('Kernel::Config');
-    my $ObjectTypeName = $ConfigObject->Get('DynamicFields::ObjectType')->{ $GetParam{ObjectType} }->{DisplayName}
-        || '';
-    my $FieldTypeName = $ConfigObject->Get('DynamicFields::Driver')->{ $GetParam{FieldType} }->{DisplayName} || '';
+    my $ObjectTypeName = $Self->{ObjectTypeConfig}->{ $GetParam{ObjectType} }->{DisplayName} || '';
+    my $FieldTypeName  = $Self->{FieldTypeConfig}->{ $GetParam{FieldType} }->{DisplayName}   || '';
 
-    my $FieldID = $ParamObject->GetParam( Param => 'ID' );
+    my $FieldID = $Self->{ParamObject}->GetParam( Param => 'ID' );
 
     if ( !$FieldID ) {
-        return $LayoutObject->ErrorScreen(
-            Message => Translatable('Need ID'),
+        return $Self->{LayoutObject}->ErrorScreen(
+            Message => "Need ID",
         );
     }
 
     # get dynamic field data
-    my $DynamicFieldData = $Kernel::OM->Get('Kernel::System::DynamicField')->DynamicFieldGet(
+    my $DynamicFieldData = $Self->{DynamicFieldObject}->DynamicFieldGet(
         ID => $FieldID,
     );
 
     # check for valid dynamic field configuration
     if ( !IsHashRefWithData($DynamicFieldData) ) {
-        return $LayoutObject->ErrorScreen(
-            Message =>
-                $LayoutObject->{LanguageObject}->Translate( 'Could not get data for dynamic field %s', $FieldID ),
+        return $Self->{LayoutObject}->ErrorScreen(
+            Message => "Could not get data for dynamic field $FieldID",
         );
     }
 
@@ -325,9 +312,6 @@ sub _Change {
 
         # set TranslatalbeValues
         $Config{TranslatableValues} = $DynamicFieldData->{Config}->{TranslatableValues};
-
-        # set TreeView
-        $Config{TreeView} = $DynamicFieldData->{Config}->{TreeView};
     }
 
     return $Self->_ShowScreen(
@@ -337,8 +321,6 @@ sub _Change {
         %Config,
         ID             => $FieldID,
         Mode           => 'Change',
-        BreadcrumbText => $LayoutObject->{LanguageObject}
-            ->Translate( 'Change %s field', $LayoutObject->{LanguageObject}->Translate($FieldTypeName) ),
         ObjectTypeName => $ObjectTypeName,
         FieldTypeName  => $FieldTypeName,
     );
@@ -349,61 +331,41 @@ sub _ChangeAction {
 
     my %Errors;
     my %GetParam;
-    my $ParamObject = $Kernel::OM->Get('Kernel::System::Web::Request');
 
     for my $Needed (qw(Name Label FieldOrder)) {
-        $GetParam{$Needed} = $ParamObject->GetParam( Param => $Needed );
+        $GetParam{$Needed} = $Self->{ParamObject}->GetParam( Param => $Needed );
         if ( !$GetParam{$Needed} ) {
             $Errors{ $Needed . 'ServerError' }        = 'ServerError';
-            $Errors{ $Needed . 'ServerErrorMessage' } = Translatable('This field is required.');
+            $Errors{ $Needed . 'ServerErrorMessage' } = 'This field is required.';
         }
     }
 
-    # get the TreeView option and set it to '0' if it is undefined
-    $GetParam{TreeView} = $ParamObject->GetParam( Param => 'TreeView' );
-    $GetParam{TreeView} = defined $GetParam{TreeView} && $GetParam{TreeView} ? '1' : '0';
-
-    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
-    my $FieldID      = $ParamObject->GetParam( Param => 'ID' );
+    my $FieldID = $Self->{ParamObject}->GetParam( Param => 'ID' );
     if ( !$FieldID ) {
-        return $LayoutObject->ErrorScreen(
-            Message => Translatable('Need ID'),
-        );
-    }
-
-    # get dynamic field data
-    my $DynamicFieldObject = $Kernel::OM->Get('Kernel::System::DynamicField');
-    my $DynamicFieldData   = $DynamicFieldObject->DynamicFieldGet(
-        ID => $FieldID,
-    );
-
-    # check for valid dynamic field configuration
-    if ( !IsHashRefWithData($DynamicFieldData) ) {
-        return $LayoutObject->ErrorScreen(
-            Message =>
-                $LayoutObject->{LanguageObject}->Translate( 'Could not get data for dynamic field %s', $FieldID ),
+        return $Self->{LayoutObject}->ErrorScreen(
+            Message => "Need ID",
         );
     }
 
     if ( $GetParam{Name} ) {
 
         # check if name is lowercase
-        if ( $GetParam{Name} !~ m{\A (?: [a-zA-Z] | \d )+ \z}xms ) {
+        if ( $GetParam{Name} !~ m{\A ( ?: [a-zA-Z] | \d )+ \z}xms ) {
 
             # add server error error class
             $Errors{NameServerError} = 'ServerError';
             $Errors{NameServerErrorMessage} =
-                Translatable('The field does not contain only ASCII letters and numbers.');
+                'The field does not contain only ASCII letters and numbers.';
         }
 
-        # get dynamic field list
-        my $DynamicFieldsList = $DynamicFieldObject->DynamicFieldList(
-            Valid      => 0,
-            ResultType => 'HASH',
-        ) || {};
-
         # check if name is duplicated
-        my %DynamicFieldsList = %{$DynamicFieldsList};
+        my %DynamicFieldsList = %{
+            $Self->{DynamicFieldObject}->DynamicFieldList(
+                Valid      => 0,
+                ResultType => 'HASH',
+                )
+        };
+
         %DynamicFieldsList = reverse %DynamicFieldsList;
 
         if (
@@ -414,31 +376,18 @@ sub _ChangeAction {
 
             # add server error class
             $Errors{NameServerError}        = 'ServerError';
-            $Errors{NameServerErrorMessage} = Translatable('There is another field with the same name.');
-        }
-
-        # if it's an internal field, it's name should not change
-        if (
-            $DynamicFieldData->{InternalField} &&
-            $DynamicFieldsList{ $GetParam{Name} } ne $FieldID
-            )
-        {
-
-            # add server error class
-            $Errors{NameServerError}        = 'ServerError';
-            $Errors{NameServerErrorMessage} = Translatable('The name for this field should not change.');
-            $Param{InternalField}           = $DynamicFieldData->{InternalField};
+            $Errors{NameServerErrorMessage} = 'There is another field with the same name.';
         }
     }
 
     if ( $GetParam{FieldOrder} ) {
 
         # check if field order is numeric and positive
-        if ( $GetParam{FieldOrder} !~ m{\A (?: \d )+ \z}xms ) {
+        if ( $GetParam{FieldOrder} !~ m{\A ( ?: \d )+ \z}xms ) {
 
             # add server error error class
             $Errors{FieldOrderServerError}        = 'ServerError';
-            $Errors{FieldOrderServerErrorMessage} = Translatable('The field must be numeric.');
+            $Errors{FieldOrderServerErrorMessage} = 'The field must be numeric.';
         }
     }
 
@@ -449,17 +398,29 @@ sub _ChangeAction {
         )
         )
     {
-        $GetParam{$ConfigParam} = $ParamObject->GetParam( Param => $ConfigParam );
+        $GetParam{$ConfigParam} = $Self->{ParamObject}->GetParam( Param => $ConfigParam );
     }
 
     # get default values
-    my @DefaultValues = $ParamObject->GetArray( Param => 'DefaultValue' );
+    my @DefaultValues = $Self->{ParamObject}->GetArray( Param => 'DefaultValue' );
     $GetParam{DefaultValue} = \@DefaultValues;
 
     # uncorrectable errors
     if ( !$GetParam{ValidID} ) {
-        return $LayoutObject->ErrorScreen(
-            Message => Translatable('Need ValidID'),
+        return $Self->{LayoutObject}->ErrorScreen(
+            Message => "Need ValidID",
+        );
+    }
+
+    # get dynamic field data
+    my $DynamicFieldData = $Self->{DynamicFieldObject}->DynamicFieldGet(
+        ID => $FieldID,
+    );
+
+    # check for valid dynamic field configuration
+    if ( !IsHashRefWithData($DynamicFieldData) ) {
+        return $Self->{LayoutObject}->ErrorScreen(
+            Message => "Could not get data for dynamic field $FieldID",
         );
     }
 
@@ -467,7 +428,7 @@ sub _ChangeAction {
 
     # set errors for possible values entries
     KEY:
-    for my $Key ( sort keys %{$PossibleValues} ) {
+    for my $Key ( keys %{$PossibleValues} ) {
 
         # check for empty original values
         if ( $Key =~ m{\A $Self->{EmptyString} (?: \d+)}smx ) {
@@ -491,34 +452,6 @@ sub _ChangeAction {
         }
     }
 
-    # Check if dynamic field is present in SysConfig setting
-    my $UpdateEntity        = $ParamObject->GetParam( Param => 'UpdateEntity' ) || '';
-    my $SysConfigObject     = $Kernel::OM->Get('Kernel::System::SysConfig');
-    my %DynamicFieldOldData = %{$DynamicFieldData};
-    my @IsDynamicFieldInSysConfig;
-    @IsDynamicFieldInSysConfig = $SysConfigObject->ConfigurationEntityCheck(
-        EntityType => 'DynamicField',
-        EntityName => $DynamicFieldData->{Name},
-    );
-    if (@IsDynamicFieldInSysConfig) {
-
-        # An entity present in SysConfig couldn't be invalidated.
-        if (
-            $Kernel::OM->Get('Kernel::System::Valid')->ValidLookup( ValidID => $GetParam{ValidID} )
-            ne 'valid'
-            )
-        {
-            $Errors{ValidIDInvalid}         = 'ServerError';
-            $Errors{ValidOptionServerError} = 'InSetting';
-        }
-
-        # In case changing name an authorization (UpdateEntity) should be send
-        elsif ( $DynamicFieldData->{Name} ne $GetParam{Name} && !$UpdateEntity ) {
-            $Errors{NameInvalid}              = 'ServerError';
-            $Errors{InSettingNameServerError} = 1;
-        }
-    }
-
     # return to change screen if errors
     if (%Errors) {
         return $Self->_ShowScreen(
@@ -534,14 +467,13 @@ sub _ChangeAction {
     # set specific config
     my $FieldConfig = {
         PossibleValues     => $PossibleValues,
-        TreeView           => $GetParam{TreeView},
         DefaultValue       => $GetParam{DefaultValue},
         PossibleNone       => $GetParam{PossibleNone},
         TranslatableValues => $GetParam{TranslatableValues},
     };
 
     # update dynamic field (FieldType and ObjectType cannot be changed; use old values)
-    my $UpdateSuccess = $DynamicFieldObject->DynamicFieldUpdate(
+    my $UpdateSuccess = $Self->{DynamicFieldObject}->DynamicFieldUpdate(
         ID         => $FieldID,
         Name       => $GetParam{Name},
         Label      => $GetParam{Label},
@@ -554,65 +486,14 @@ sub _ChangeAction {
     );
 
     if ( !$UpdateSuccess ) {
-        return $LayoutObject->ErrorScreen(
-            Message => $LayoutObject->{LanguageObject}->Translate( 'Could not update the field %s', $GetParam{Name} ),
+        return $Self->{LayoutObject}->ErrorScreen(
+            Message => "Could not update the field $GetParam{Name}",
         );
     }
 
-    if (
-        @IsDynamicFieldInSysConfig
-        && $DynamicFieldOldData{Name} ne $GetParam{Name}
-        && $UpdateEntity
-        )
-    {
-        SETTING:
-        for my $SettingName (@IsDynamicFieldInSysConfig) {
-
-            my %Setting = $SysConfigObject->SettingGet(
-                Name => $SettingName,
-            );
-
-            next SETTING if !IsHashRefWithData( \%Setting );
-
-            $Setting{EffectiveValue} =~ s/$DynamicFieldOldData{Name}/$GetParam{Name}/g;
-
-            my $ExclusiveLockGUID = $SysConfigObject->SettingLock(
-                Name   => $Setting{Name},
-                Force  => 1,
-                UserID => $Self->{UserID}
-            );
-            $Setting{ExclusiveLockGUID} = $ExclusiveLockGUID;
-
-            my %UpdateSuccess = $SysConfigObject->SettingUpdate(
-                %Setting,
-                UserID => $Self->{UserID},
-            );
-        }
-
-        $SysConfigObject->ConfigurationDeploy(
-            Comments      => "DynamicField name change",
-            DirtySettings => \@IsDynamicFieldInSysConfig,
-            UserID        => $Self->{UserID},
-            Force         => 1,
-        );
-    }
-
-    # if the user would like to continue editing the dynamic field, just redirect to the change screen
-    if (
-        defined $ParamObject->GetParam( Param => 'ContinueAfterSave' )
-        && ( $ParamObject->GetParam( Param => 'ContinueAfterSave' ) eq '1' )
-        )
-    {
-        return $LayoutObject->Redirect(
-            OP =>
-                "Action=$Self->{Action};Subaction=Change;ObjectType=$DynamicFieldData->{ObjectType};FieldType=$DynamicFieldData->{FieldType};ID=$FieldID"
-        );
-    }
-    else {
-
-        # otherwise return to overview
-        return $LayoutObject->Redirect( OP => "Action=AdminDynamicField" );
-    }
+    return $Self->{LayoutObject}->Redirect(
+        OP => "Action=AdminDynamicField",
+    );
 }
 
 sub _ShowScreen {
@@ -627,29 +508,25 @@ sub _ShowScreen {
 
     $Param{DeletedString} = $Self->{DeletedString};
 
-    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
-
     # header
-    my $Output = $LayoutObject->Header();
-    $Output .= $LayoutObject->NavigationBar();
+    my $Output = $Self->{LayoutObject}->Header();
+    $Output .= $Self->{LayoutObject}->NavigationBar();
 
     # get all fields
-    my $DynamicFieldList = $Kernel::OM->Get('Kernel::System::DynamicField')->DynamicFieldListGet(
+    my $DynamicFieldList = $Self->{DynamicFieldObject}->DynamicFieldListGet(
         Valid => 0,
     );
 
     # get the list of order numbers (is already sorted).
     my @DynamicfieldOrderList;
-    my %DynamicfieldNamesList;
     for my $Dynamicfield ( @{$DynamicFieldList} ) {
         push @DynamicfieldOrderList, $Dynamicfield->{FieldOrder};
-        $DynamicfieldNamesList{ $Dynamicfield->{FieldOrder} } = $Dynamicfield->{Label};
     }
 
     # when adding we need to create an extra order number for the new field
     if ( $Param{Mode} eq 'Add' ) {
 
-        # get the last element from the order list and add 1
+        # get the last element form the order list and add 1
         my $LastOrderNumber = $DynamicfieldOrderList[-1];
         $LastOrderNumber++;
 
@@ -657,38 +534,24 @@ sub _ShowScreen {
         push @DynamicfieldOrderList, $LastOrderNumber;
     }
 
-    # show the names of the other fields to ease ordering
-    my %OrderNamesList;
-    my $CurrentlyText = $LayoutObject->{LanguageObject}->Translate('Currently') . ': ';
-    for my $OrderNumber ( sort @DynamicfieldOrderList ) {
-        $OrderNamesList{$OrderNumber} = $OrderNumber;
-        if ( $DynamicfieldNamesList{$OrderNumber} && $OrderNumber ne $Param{FieldOrder} ) {
-            $OrderNamesList{$OrderNumber} = $OrderNumber . ' - '
-                . $CurrentlyText
-                . $DynamicfieldNamesList{$OrderNumber};
-        }
-    }
-
-    my $DynamicFieldOrderStrg = $LayoutObject->BuildSelection(
-        Data          => \%OrderNamesList,
+    my $DynamicFieldOrderSrtg = $Self->{LayoutObject}->BuildSelection(
+        Data          => \@DynamicfieldOrderList,
         Name          => 'FieldOrder',
         SelectedValue => $Param{FieldOrder} || 1,
         PossibleNone  => 0,
-        Translation   => 0,
-        Sort          => 'NumericKey',
-        Class         => 'Modernize W75pc Validate_Number',
+        Class         => 'W50pc Validate_Number',
     );
 
-    my %ValidList = $Kernel::OM->Get('Kernel::System::Valid')->ValidList();
+    my %ValidList = $Self->{ValidObject}->ValidList();
 
     # create the Validity select
-    my $ValidityStrg = $LayoutObject->BuildSelection(
+    my $ValidityStrg = $Self->{LayoutObject}->BuildSelection(
         Data         => \%ValidList,
         Name         => 'ValidID',
         SelectedID   => $Param{ValidID} || 1,
         PossibleNone => 0,
         Translation  => 1,
-        Class        => 'Modernize W50pc',
+        Class        => 'W50pc',
     );
 
     # define as 0 to get the real value in the HTML
@@ -725,7 +588,7 @@ sub _ShowScreen {
 
                 # set the error class
                 $KeyError     = 'ServerError';
-                $KeyErrorStrg = Translatable('This field is required.');
+                $KeyErrorStrg = 'This field is required.'
             }
 
             # check for errors on original value (duplicate)
@@ -733,11 +596,12 @@ sub _ShowScreen {
 
                 # if the original value was empty it has been changed in _GetParams to a predefined
                 # string and need to be set to the original value again
-                $KeyClone = $Param{'PossibleValueErrors'}->{'KeyDuplicateError'}->{$Key};
+                $KeyClone
+                    = $Param{'PossibleValueErrors'}->{'KeyDuplicateError'}->{$Key};
 
                 # set the error class
                 $KeyError     = 'ServerError';
-                $KeyErrorStrg = Translatable('This field value is duplicated.');
+                $KeyErrorStrg = 'This field value is duplicated.'
             }
 
             # check for error on value
@@ -749,11 +613,11 @@ sub _ShowScreen {
         }
 
         # create a value map row
-        $LayoutObject->Block(
+        $Self->{LayoutObject}->Block(
             Name => 'ValueRow',
             Data => {
                 KeyError     => $KeyError,
-                KeyErrorStrg => $KeyErrorStrg || Translatable('This field is required.'),
+                KeyErrorStrg => $KeyErrorStrg || 'This field is required.',
                 Key          => $KeyClone,
                 ValueCounter => $ValueCounter,
                 Value        => $PossibleValues{$Key},
@@ -763,7 +627,7 @@ sub _ShowScreen {
     }
 
     # create the possible values template
-    $LayoutObject->Block(
+    $Self->{LayoutObject}->Block(
         Name => 'ValueTemplate',
         Data => {
             %Param,
@@ -773,16 +637,16 @@ sub _ShowScreen {
     # check and build the Default Value list based on Possible Values
     my %DefaultValuesList;
     POSSIBLEVALUE:
-    for my $ValueItem ( sort keys %PossibleValues ) {
+    for my $ValueItem ( keys %PossibleValues ) {
         next POSSIBLEVALUE if !defined $ValueItem;
         next POSSIBLEVALUE if !defined $PossibleValues{$ValueItem};
-        $DefaultValuesList{$ValueItem} = $PossibleValues{$ValueItem};
+        $DefaultValuesList{$ValueItem} = $PossibleValues{$ValueItem}
     }
 
     my $DefaultValue = ( defined $Param{DefaultValue} ? $Param{DefaultValue} : '' );
 
     # create the default value select
-    my $DefaultValueStrg = $LayoutObject->BuildSelection(
+    my $DefaultValueStrg = $Self->{LayoutObject}->BuildSelection(
         Data         => \%DefaultValuesList,
         Name         => 'DefaultValue',
         SelectedID   => $DefaultValue,
@@ -794,121 +658,50 @@ sub _ShowScreen {
 
         #        # Multiple selections, now is going to be supported
         Multiple => 1,
-        Class    => 'Modernize W50pc',
+        Class    => 'W50pc',
     );
 
     my $PossibleNone = $Param{PossibleNone} || '0';
 
     # create translatable values option list
-    my $PossibleNoneStrg = $LayoutObject->BuildSelection(
+    my $PossibleNoneStrg = $Self->{LayoutObject}->BuildSelection(
         Data => {
-            0 => Translatable('No'),
-            1 => Translatable('Yes'),
+            0 => 'No',
+            1 => 'Yes',
         },
         Name       => 'PossibleNone',
         SelectedID => $PossibleNone,
-        Class      => 'Modernize W50pc',
+        Class      => 'W50pc',
     );
 
     my $TranslatableValues = $Param{TranslatableValues} || '0';
 
     # create translatable values option list
-    my $TranslatableValuesStrg = $LayoutObject->BuildSelection(
+    my $TranslatableValuesStrg = $Self->{LayoutObject}->BuildSelection(
         Data => {
-            0 => Translatable('No'),
-            1 => Translatable('Yes'),
+            0 => 'No',
+            1 => 'Yes',
         },
         Name       => 'TranslatableValues',
         SelectedID => $TranslatableValues,
-        Class      => 'Modernize W50pc',
+        Class      => 'W50pc',
     );
 
-    my $TreeView = $Param{TreeView} || '0';
-
-    # create treeview option list
-    my $TreeViewStrg = $LayoutObject->BuildSelection(
-        Data => {
-            0 => Translatable('No'),
-            1 => Translatable('Yes'),
-        },
-        Name       => 'TreeView',
-        SelectedID => $TreeView,
-        Class      => 'Modernize W50pc',
-    );
-
-    my $ReadonlyInternalField = '';
-
-    # Internal fields can not be deleted and name should not change.
-    if ( $Param{InternalField} ) {
-        $LayoutObject->Block(
-            Name => 'InternalField',
-            Data => {%Param},
-        );
-        $ReadonlyInternalField = 'readonly="readonly"';
-    }
-
-    my $DynamicFieldName = $Param{Name};
-
-    # Add warning in case the DynamicField belongs a SysConfig setting.
-    my $SysConfigObject = $Kernel::OM->Get('Kernel::System::SysConfig');
-
-    # In case dirty setting disable form
-    my $IsDirtyConfig = 0;
-    my @IsDirtyResult = $SysConfigObject->ConfigurationDirtySettingsList();
-    my %IsDirtyList   = map { $_ => 1 } @IsDirtyResult;
-
-    my @IsDynamicFieldInSysConfig = $SysConfigObject->ConfigurationEntityCheck(
-        EntityType => 'DynamicField',
-        EntityName => $DynamicFieldName // '',
-    );
-
-    if (@IsDynamicFieldInSysConfig) {
-        $LayoutObject->Block(
-            Name => 'DynamicFieldInSysConfig',
-            Data => {
-                OldName => $DynamicFieldName,
-            },
-        );
-        for my $SettingName (@IsDynamicFieldInSysConfig) {
-            $LayoutObject->Block(
-                Name => 'DynamicFieldInSysConfigRow',
-                Data => {
-                    SettingName => $SettingName,
-                },
-            );
-
-            # Verify if dirty setting
-            if ( $IsDirtyList{$SettingName} ) {
-                $IsDirtyConfig = 1;
-            }
-
-        }
-    }
-
-    if ($IsDirtyConfig) {
-        $LayoutObject->Block(
-            Name => 'DynamicFieldInSysConfigDirty',
-            ,
-        );
-    }
-
     # generate output
-    $Output .= $LayoutObject->Output(
+    $Output .= $Self->{LayoutObject}->Output(
         TemplateFile => 'AdminDynamicFieldMultiselect',
         Data         => {
             %Param,
             ValidityStrg           => $ValidityStrg,
-            DynamicFieldOrderStrg  => $DynamicFieldOrderStrg,
+            DynamicFieldOrderSrtg  => $DynamicFieldOrderSrtg,
             ValueCounter           => $ValueCounter,
             DefaultValueStrg       => $DefaultValueStrg,
             PossibleNoneStrg       => $PossibleNoneStrg,
-            TreeViewStrg           => $TreeViewStrg,
             TranslatableValuesStrg => $TranslatableValuesStrg,
-            ReadonlyInternalField  => $ReadonlyInternalField,
-        }
+            }
     );
 
-    $Output .= $LayoutObject->Footer();
+    $Output .= $Self->{LayoutObject}->Footer();
 
     return $Output;
 }
@@ -916,20 +709,19 @@ sub _ShowScreen {
 sub _GetPossibleValues {
     my ( $Self, %Param ) = @_;
 
-    my $ParamObject = $Kernel::OM->Get('Kernel::System::Web::Request');
+    my $PossibleValueConfig;
 
     # get parameters from web browser
     # get ValueCounters
-    my $ValueCounter          = $ParamObject->GetParam( Param => 'ValueCounter' ) || 0;
+    my $ValueCounter          = $Self->{ParamObject}->GetParam( Param => 'ValueCounter' ) || 0;
     my $EmptyValueCounter     = 0;
     my $DuplicateValueCounter = 0;
-    my $PossibleValueConfig;
 
     # get possible values
     my $Values;
     VALUEINDEX:
     for my $ValueIndex ( 1 .. $ValueCounter ) {
-        my $Key = $ParamObject->GetParam( Param => 'Key' . '_' . $ValueIndex );
+        my $Key = $Self->{ParamObject}->GetParam( Param => 'Key' . '_' . $ValueIndex );
         $Key = ( defined $Key ? $Key : '' );
 
         # check if key was deleted by the user and skip it
@@ -951,7 +743,7 @@ sub _GetPossibleValues {
             $DuplicateValueCounter++;
         }
 
-        my $Value = $ParamObject->GetParam( Param => 'Value' . '_' . $ValueIndex );
+        my $Value = $Self->{ParamObject}->GetParam( Param => 'Value' . '_' . $ValueIndex );
         $Value = ( defined $Value ? $Value : '' );
         $PossibleValueConfig->{$Key} = $Value;
     }
