@@ -1,8 +1,8 @@
 # --
 # Kernel/Modules/AgentTicketBounce.pm - to bounce articles of tickets
-# Copyright (C) 2001-2010 OTRS AG, http://otrs.org/
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
-# $Id: AgentTicketBounce.pm,v 1.49 2010/12/22 13:14:45 mb Exp $
+# $Id: AgentTicketBounce.pm,v 1.56 2012-03-05 09:51:33 mg Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -17,11 +17,13 @@ use warnings;
 use Kernel::System::State;
 use Kernel::System::SystemAddress;
 use Kernel::System::CustomerUser;
+use Kernel::System::CheckItem;
 use Kernel::System::TemplateGenerator;
+use Kernel::System::VariableCheck qw(:all);
 use Mail::Address;
 
 use vars qw($VERSION);
-$VERSION = qw($Revision: 1.49 $) [1];
+$VERSION = qw($Revision: 1.56 $) [1];
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -43,6 +45,7 @@ sub new {
     # needed objects
     $Self->{StateObject}        = Kernel::System::State->new(%Param);
     $Self->{CustomerUserObject} = Kernel::System::CustomerUser->new(%Param);
+    $Self->{CheckItemObject}    = Kernel::System::CheckItem->new(%Param);
     $Self->{SystemAddress}      = Kernel::System::SystemAddress->new(%Param);
     $Self->{ArticleID}          = $Self->{ParamObject}->GetParam( Param => 'ArticleID' ) || '';
     $Self->{Config}             = $Self->{ConfigObject}->Get("Ticket::Frontend::$Self->{Action}");
@@ -78,6 +81,25 @@ sub Run {
         return $Self->{LayoutObject}->NoPermission( WithHeader => 'yes' );
     }
 
+    # get ACL restrictions
+    $Self->{TicketObject}->TicketAcl(
+        Data          => '-',
+        TicketID      => $Self->{TicketID},
+        ReturnType    => 'Action',
+        ReturnSubType => '-',
+        UserID        => $Self->{UserID},
+    );
+    my %AclAction = $Self->{TicketObject}->TicketAclActionData();
+
+    # check if ACL resctictions if exist
+    if ( IsHashRefWithData( \%AclAction ) ) {
+
+        # show error screen if ACL prohibits this action
+        if ( defined $AclAction{ $Self->{Action} } && $AclAction{ $Self->{Action} } eq '0' ) {
+            return $Self->{LayoutObject}->NoPermission( WithHeader => 'yes' );
+        }
+    }
+
     # get lock state && write (lock) permissions
     if ( $Self->{Config}->{RequiredLock} ) {
         if ( !$Self->{TicketObject}->TicketLockGet( TicketID => $Self->{TicketID} ) ) {
@@ -112,7 +134,7 @@ sub Run {
                     Type  => 'Small',
                 );
                 $Output .= $Self->{LayoutObject}->Warning(
-                    Message => "Sorry, you need to be the owner to do this action!",
+                    Message => "Sorry, you need to be the ticket owner to perform this action.",
                     Comment => 'Please change the owner first.',
                 );
                 $Output .= $Self->{LayoutObject}->Footer(
@@ -146,7 +168,17 @@ sub Run {
         }
 
         # get article data
-        my %Article = $Self->{TicketObject}->ArticleGet( ArticleID => $Self->{ArticleID}, );
+        my %Article = $Self->{TicketObject}->ArticleGet(
+            ArticleID     => $Self->{ArticleID},
+            DynamicFields => 0,
+        );
+
+        # Check if article is from the same TicketID as we checked permissions for.
+        if ( $Article{TicketID} ne $Self->{TicketID} ) {
+            return $Self->{LayoutObject}->ErrorScreen(
+                Message => "Article does not belong to ticket $Self->{TicketID}!",
+            );
+        }
 
         # prepare to (ReplyTo!) ...
         if ( $Article{ReplyTo} ) {
@@ -272,23 +304,52 @@ $Param{Signature}";
         # check forward email address
         if ( !$Param{BounceTo} ) {
             $Error{'BounceToInvalid'} = 'ServerError';
+            $Self->{LayoutObject}->Block( Name => 'BounceToCustomerGenericServerErrorMsg' );
         }
         for my $Email ( Mail::Address->parse( $Param{BounceTo} ) ) {
             my $Address = $Email->address();
             if ( $Self->{SystemAddress}->SystemAddressIsLocalAddress( Address => $Address ) ) {
+                $Self->{LayoutObject}->Block( Name => 'BounceToCustomerGenericServerErrorMsg' );
+                $Error{'BounceToInvalid'} = 'ServerError';
+            }
+
+            # check email address
+            elsif ( !$Self->{CheckItemObject}->CheckEmail( Address => $Address ) ) {
+                my $BounceToErrorMsg =
+                    'BounceTo'
+                    . $Self->{CheckItemObject}->CheckErrorType()
+                    . 'ServerErrorMsg';
+                $Self->{LayoutObject}->Block( Name => $BounceToErrorMsg );
                 $Error{'BounceToInvalid'} = 'ServerError';
             }
         }
 
-        if ( !$Param{To} ) {
-            $Error{'ToInvalid'} = 'ServerError';
-        }
+        if ( $Param{InformSender} ) {
+            if ( !$Param{To} ) {
+                $Error{'ToInvalid'} = 'ServerError';
+                $Self->{LayoutObject}->Block( Name => 'ToCustomerGenericServerErrorMsg' );
+            }
+            else {
 
-        if ( !$Param{Subject} ) {
-            $Error{'SubjectInvalid'} = 'ServerError';
-        }
-        if ( !$Param{Body} ) {
-            $Error{'BodyInvalid'} = 'ServerError';
+                # check email address(es)
+                for my $Email ( Mail::Address->parse( $Param{To} ) ) {
+                    if ( !$Self->{CheckItemObject}->CheckEmail( Address => $Email->address() ) ) {
+                        my $ToErrorMsg =
+                            'To'
+                            . $Self->{CheckItemObject}->CheckErrorType()
+                            . 'ServerErrorMsg';
+                        $Self->{LayoutObject}->Block( Name => $ToErrorMsg );
+                        $Error{'ToInvalid'} = 'ServerError';
+                    }
+                }
+            }
+
+            if ( !$Param{Subject} ) {
+                $Error{'SubjectInvalid'} = 'ServerError';
+            }
+            if ( !$Param{Body} ) {
+                $Error{'BodyInvalid'} = 'ServerError';
+            }
         }
 
         #check for error
@@ -306,9 +367,9 @@ $Param{Signature}";
                 $NextStates{''} = '-';
             }
             $Param{NextStatesStrg} = $Self->{LayoutObject}->BuildSelection(
-                Data     => \%NextStates,
-                Name     => 'BounceStateID',
-                Selected => $Param{BounceStateID},
+                Data       => \%NextStates,
+                Name       => 'BounceStateID',
+                SelectedID => $Param{BounceStateID},
             );
 
             # add rich text editor
@@ -318,7 +379,16 @@ $Param{Signature}";
                 );
             }
 
+            # prepare bounce tags if body is rich text
+            if ( $Self->{LayoutObject}->{BrowserRichText} ) {
+
+                # prepare bounce tags
+                $Param{Body} =~ s/&lt;OTRS_TICKET&gt;/&amp;lt;OTRS_TICKET&amp;gt;/gi;
+                $Param{Body} =~ s/&lt;OTRS_BOUNCE_TO&gt;/&amp;lt;OTRS_BOUNCE_TO&amp;gt;/gi;
+            }
+
             $Param{InformationFormat} = $Param{Body};
+            $Param{InformSenderChecked} = $Param{InformSender} ? 'checked="checked"' : '';
 
             my $Output = $Self->{LayoutObject}->Header(
                 Type => 'Small',
@@ -400,37 +470,39 @@ $Param{Signature}";
             }
         }
 
-        # set state
-        my %StateData = $Self->{TicketObject}->{StateObject}->StateGet(
-            ID => $Param{BounceStateID},
-        );
-        $Self->{TicketObject}->TicketStateSet(
-            TicketID  => $Self->{TicketID},
-            ArticleID => $Self->{ArticleID},
-            StateID   => $Param{BounceStateID},
-            UserID    => $Self->{UserID},
-        );
+        # check if there is a chosen bounce state id
+        if ( $Param{BounceStateID} ) {
 
-        # should i set an unlock?
-        if ( $StateData{TypeName} =~ /^close/i ) {
-            $Self->{TicketObject}->TicketLockSet(
-                TicketID => $Self->{TicketID},
-                Lock     => 'unlock',
-                UserID   => $Self->{UserID},
+            # set state
+            my %StateData = $Self->{TicketObject}->{StateObject}->StateGet(
+                ID => $Param{BounceStateID},
+            );
+            $Self->{TicketObject}->TicketStateSet(
+                TicketID  => $Self->{TicketID},
+                ArticleID => $Self->{ArticleID},
+                StateID   => $Param{BounceStateID},
+                UserID    => $Self->{UserID},
             );
-        }
 
-        # redirect
-        if ( $StateData{TypeName} =~ /^close/i ) {
-            return $Self->{LayoutObject}->PopupClose(
-                URL => ( $Self->{LastScreenOverview} || 'Action=AgentDashboard' ),
-            );
-        }
-        else {
-            return $Self->{LayoutObject}->PopupClose(
-                URL => ( $Self->{LastScreenView} || 'Action=AgentDashboard' )
-            );
+            # should i set an unlock?
+            if ( $StateData{TypeName} =~ /^close/i ) {
+                $Self->{TicketObject}->TicketLockSet(
+                    TicketID => $Self->{TicketID},
+                    Lock     => 'unlock',
+                    UserID   => $Self->{UserID},
+                );
+            }
+
+            # redirect
+            if ( $StateData{TypeName} =~ /^close/i ) {
+                return $Self->{LayoutObject}->PopupClose(
+                    URL => ( $Self->{LastScreenOverview} || 'Action=AgentDashboard' ),
+                );
+            }
         }
+        return $Self->{LayoutObject}->PopupClose(
+            URL => ( $Self->{LastScreenView} || 'Action=AgentDashboard' )
+        );
     }
     return $Self->{LayoutObject}->ErrorScreen(
         Message => 'Wrong Subaction!!',
