@@ -1,8 +1,8 @@
 # --
 # Kernel/System/GenericAgent.pm - generic agent system module
-# Copyright (C) 2001-2012 OTRS AG, http://otrs.org/
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
-# $Id: GenericAgent.pm,v 1.70.2.2 2012/02/15 22:45:18 cg Exp $
+# $Id: GenericAgent.pm,v 1.81 2012-02-14 08:40:37 mg Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -14,8 +14,13 @@ package Kernel::System::GenericAgent;
 use strict;
 use warnings;
 
+use Kernel::System::DynamicField;
+use Kernel::System::DynamicField::Backend;
+use Kernel::System::State;
+use Kernel::System::VariableCheck qw(:all);
+
 use vars qw($VERSION);
-$VERSION = qw($Revision: 1.70.2.2 $) [1];
+$VERSION = qw($Revision: 1.81 $) [1];
 
 =head1 NAME
 
@@ -105,6 +110,25 @@ sub new {
         $Self->{$_} = $Param{$_} || die "Got no $_!";
     }
 
+    # create additional objects
+    $Self->{DynamicFieldObject} = Kernel::System::DynamicField->new(%Param);
+    $Self->{BackendObject}      = Kernel::System::DynamicField::Backend->new(%Param);
+    $Self->{StateObject}        = Kernel::System::State->new(%Param);
+
+    my %PendingStates = $Self->{StateObject}->StateGetStatesByType(
+        StateType => [ 'pending auto', 'pending reminder' ],
+        Result => 'HASH',
+    );
+
+    $Self->{PendingStateList} = \%PendingStates || {};
+    $Self->{CurrentSystemTime} = $Self->{TimeObject}->SystemTime();
+
+    # get the dynamic fields for ticket object
+    $Self->{DynamicField} = $Self->{DynamicFieldObject}->DynamicFieldListGet(
+        Valid      => 1,
+        ObjectType => ['Ticket'],
+    );
+
     # debug
     $Self->{Debug} = $Param{Debug} || 0;
 
@@ -130,6 +154,7 @@ sub new {
         OwnerIDs                => 'ARRAY',
         LockIDs                 => 'ARRAY',
         TypeIDs                 => 'ARRAY',
+        ResponsibleIDs          => 'ARRAY',
         ServiceIDs              => 'ARRAY',
         SLAIDs                  => 'ARRAY',
         NewTitle                => 'SCALAR',
@@ -141,6 +166,7 @@ sub new {
         NewOwnerID              => 'SCALAR',
         NewLockID               => 'SCALAR',
         NewTypeID               => 'SCALAR',
+        NewResponsibleID        => 'SCALAR',
         NewServiceID            => 'SCALAR',
         NewSLAID                => 'SCALAR',
         ScheduleLastRun         => 'SCALAR',
@@ -153,14 +179,14 @@ sub new {
 
     # add time attributes
     for my $Type (
-        qw(Time CloseTime TimePending EscalationTime EscalationResponseTime EscalationUpdateTime EscalationSolutionTime)
+        qw(Time ChangeTime CloseTime TimePending EscalationTime EscalationResponseTime EscalationUpdateTime EscalationSolutionTime)
         )
     {
         my $Key = $Type . 'SearchType';
         $Map{$Key} = 'SCALAR';
     }
     for my $Type (
-        qw(TicketCreate TicketClose TicketPending TicketEscalation TicketEscalationResponse TicketEscalationUpdate TicketEscalationSolution)
+        qw(TicketCreate TicketChange TicketClose TicketPending TicketEscalation TicketEscalationResponse TicketEscalationUpdate TicketEscalationSolution)
         )
     {
         for my $Attribute (
@@ -172,13 +198,23 @@ sub new {
         }
     }
 
-    # add free text attributes
-    for my $Type ( 1 .. 16 ) {
-        my $Key   = 'TicketFreeKey' . $Type;
-        my $Value = 'TicketFreeText' . $Type;
+    # Add Dynamic Fields attributes
+    DYNAMICFIELD:
+    for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
+        next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
 
-        $Map{$Key}   = 'ARRAY';
-        $Map{$Value} = 'ARRAY';
+        # get the field type of the dynamic fields for edit and search
+        my $FieldValueType = $Self->{BackendObject}->TemplateValueTypeGet(
+            DynamicFieldConfig => $DynamicFieldConfig,
+            FieldType          => 'All',
+        );
+
+        # Add field type to Map
+        if ( IsHashRefWithData($FieldValueType) ) {
+            for my $FieldName ( keys %{$FieldValueType} ) {
+                $Map{$FieldName} = $FieldValueType->{$FieldName};
+            }
+        }
     }
 
     $Self->{Map} = \%Map;
@@ -213,6 +249,7 @@ sub JobRun {
 
     # get job from param
     my %Job;
+    my %DynamicFieldSearchTemplate;
     if ( $Param{Config} ) {
         %Job = %{ $Param{Config} };
 
@@ -246,7 +283,19 @@ sub JobRun {
                 $Job{New}->{$NewKey} = $DBJobRaw{$Key};
             }
             else {
-                $Job{$Key} = $DBJobRaw{$Key};
+
+                # skip dynamic fields
+                if ( $Key !~ m{ DynamicField_ }xms ) {
+                    $Job{$Key} = $DBJobRaw{$Key};
+                }
+            }
+
+            # convert dynamic fields
+            if ( $Key =~ m{ \A DynamicField_ }xms ) {
+                $Job{New}->{$Key} = $DBJobRaw{$Key};
+            }
+            elsif ( $Key =~ m{ \A Search_DynamicField_ }xms ) {
+                $DynamicFieldSearchTemplate{$Key} = $DBJobRaw{$Key};
             }
         }
         if ( exists $Job{SearchInArchive} && $Job{SearchInArchive} eq 'ArchivedTickets' ) {
@@ -257,20 +306,51 @@ sub JobRun {
         }
     }
 
+    # set dynamic fields search parameters
+    my %DynamicFieldSearchParameters;
+    DYNAMICFIELD:
+    for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
+        next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
+        next DYNAMICFIELD
+            if !$DynamicFieldSearchTemplate{ 'Search_DynamicField_' . $DynamicFieldConfig->{Name} };
+
+        # extract the dynamic field value form the Generic Agent Job
+        my $SearchParameter = $Self->{BackendObject}->SearchFieldParameterBuild(
+            DynamicFieldConfig => $DynamicFieldConfig,
+            Profile            => \%DynamicFieldSearchTemplate,
+        );
+
+        # set search parameter
+        if ( defined $SearchParameter ) {
+            $DynamicFieldSearchParameters{ 'DynamicField_' . $DynamicFieldConfig->{Name} }
+                = $SearchParameter->{Parameter};
+        }
+    }
+
     my %Tickets;
 
     # escalation tickets
     if ( $Job{Escalation} ) {
-        if ( !$Job{Queue} ) {
-            my @Tickets = $Self->{TicketObject}->GetOverTimeTickets();
-            for (@Tickets) {
+
+        # Find all tickets which will escalate within the next five days.
+        #   The notification module will determine if a notification must be sent out or not.
+        my @Tickets = $Self->{TicketObject}->TicketSearch(
+            Result                           => 'ARRAY',
+            Limit                            => 100,
+            TicketEscalationTimeOlderMinutes => -( 5 * 24 * 60 ),
+            Permission                       => 'rw',
+            UserID                           => $Param{UserID} || 1,
+        );
+
+        for (@Tickets) {
+            if ( !$Job{Queue} ) {
                 $Tickets{$_} = $Self->{TicketObject}->TicketNumberLookup( TicketID => $_ );
             }
-        }
-        else {
-            my @Tickets = $Self->{TicketObject}->GetOverTimeTickets();
-            for (@Tickets) {
-                my %Ticket = $Self->{TicketObject}->TicketGet( TicketID => $_ );
+            else {
+                my %Ticket = $Self->{TicketObject}->TicketGet(
+                    TicketID      => $_,
+                    DynamicFields => 0,
+                );
                 if ( $Ticket{Queue} eq $Job{Queue} ) {
                     $Tickets{$_} = $Ticket{TicketNumber};
                 }
@@ -291,6 +371,7 @@ sub JobRun {
             %Tickets = (
                 $Self->{TicketObject}->TicketSearch(
                     %Job,
+                    %DynamicFieldSearchParameters,
                     ConditionInline => 1,
                     StateType       => $Type,
                     Limit           => $Param{Limit} || 4000,
@@ -307,6 +388,7 @@ sub JobRun {
                 %Tickets = (
                     $Self->{TicketObject}->TicketSearch(
                         %Job,
+                        %DynamicFieldSearchParameters,
                         ConditionInline => 1,
                         Queues          => [$_],
                         StateType       => $Type,
@@ -321,6 +403,7 @@ sub JobRun {
             %Tickets = (
                 $Self->{TicketObject}->TicketSearch(
                     %Job,
+                    %DynamicFieldSearchParameters,
                     ConditionInline => 1,
                     StateType       => $Type,
                     Queues          => [ $Job{Queue} ],
@@ -331,7 +414,10 @@ sub JobRun {
             );
         }
         for ( keys %Tickets ) {
-            my %Ticket = $Self->{TicketObject}->TicketGet( TicketID => $_ );
+            my %Ticket = $Self->{TicketObject}->TicketGet(
+                TicketID      => $_,
+                DynamicFields => 0,
+            );
             if ( $Ticket{UntilTime} > 1 ) {
                 delete $Tickets{$_};
             }
@@ -350,7 +436,12 @@ sub JobRun {
                 }
             }
 
-            # log no search attribut
+            # also search in Dynamic fields search attributes
+            for my $DynamicFieldName ( keys %DynamicFieldSearchParameters ) {
+                $Count++;
+            }
+
+            # log no search attribute
             if ( !$Count ) {
                 $Self->{LogObject}->Log(
                     Priority => 'error',
@@ -366,6 +457,7 @@ sub JobRun {
             }
             %Tickets = $Self->{TicketObject}->TicketSearch(
                 %Job,
+                %DynamicFieldSearchParameters,
                 ConditionInline => 1,
                 Limit           => $Param{Limit} || 4000,
                 UserID          => $Param{UserID},
@@ -379,6 +471,7 @@ sub JobRun {
                 %Tickets = (
                     $Self->{TicketObject}->TicketSearch(
                         %Job,
+                        %DynamicFieldSearchParameters,
                         ConditionInline => 1,
                         Queues          => [$_],
                         Limit           => $Param{Limit} || 4000,
@@ -391,6 +484,7 @@ sub JobRun {
         else {
             %Tickets = $Self->{TicketObject}->TicketSearch(
                 %Job,
+                %DynamicFieldSearchParameters,
                 ConditionInline => 1,
                 Queues          => [ $Job{Queue} ],
                 Limit           => $Param{Limit} || 4000,
@@ -482,6 +576,7 @@ sub JobGet {
     # get time settings
     my %Map = (
         TicketCreate             => 'Time',
+        TicketChange             => 'ChangeTime',
         TicketClose              => 'CloseTime',
         TicketPending            => 'TimePending',
         TicketEscalation         => 'EscalationTime',
@@ -490,7 +585,7 @@ sub JobGet {
         TicketEscalationSolution => 'EscalationSolutionTime',
     );
     for my $Type (
-        qw(TicketCreate TicketClose TicketPending TicketEscalation TicketEscalationResponse TicketEscalationUpdate TicketEscalationSolution)
+        qw(TicketCreate TicketChange TicketClose TicketPending TicketEscalation TicketEscalationResponse TicketEscalationUpdate TicketEscalationSolution)
         )
     {
         my $SearchType = $Map{$Type} . 'SearchType';
@@ -770,7 +865,7 @@ sub _JobRunTicket {
                 || 'Note',
             Body => $Param{Config}->{New}->{Note}->{Body} || $Param{Config}->{New}->{NoteBody},
             MimeType       => 'text/plain',
-            Charset        => $Self->{ConfigObject}->Get('DefaultCharset'),
+            Charset        => 'utf-8',
             UserID         => $Param{UserID},
             HistoryType    => 'AddNote',
             HistoryComment => 'Generic Agent note added.',
@@ -788,6 +883,8 @@ sub _JobRunTicket {
         }
     }
 
+    my $IsPendingState;
+
     # set new state
     if ( $Param{Config}->{New}->{State} ) {
         if ( $Self->{NoticeSTDOUT} ) {
@@ -798,6 +895,9 @@ sub _JobRunTicket {
             UserID   => $Param{UserID},
             State    => $Param{Config}->{New}->{State},
         );
+
+        $IsPendingState
+            = grep { $_ eq $Param{Config}->{New}->{State} } values %{ $Self->{PendingStateList} };
     }
     if ( $Param{Config}->{New}->{StateID} ) {
         if ( $Self->{NoticeSTDOUT} ) {
@@ -808,6 +908,43 @@ sub _JobRunTicket {
             UserID   => $Param{UserID},
             StateID  => $Param{Config}->{New}->{StateID},
         );
+
+        $IsPendingState
+            = grep { $_ == $Param{Config}->{New}->{StateID} } keys %{ $Self->{PendingStateList} };
+    }
+
+    # set pending time, if new state is pending state
+    if ($IsPendingState) {
+        if ( $Param{Config}->{New}->{PendingTime} ) {
+
+            # pending time
+            my $PendingTime = $Param{Config}->{New}->{PendingTime};
+
+            # calculate pending time based on hours, minutes, years...
+            if ( $Param{Config}->{New}->{PendingTimeType} ) {
+                $PendingTime *= $Param{Config}->{New}->{PendingTimeType};
+            }
+
+            # add systemtime
+            $PendingTime += $Self->{CurrentSystemTime};
+
+            # get date
+            my ( $Sec, $Min, $Hour, $Day, $Month, $Year, $WeekDay )
+                = $Self->{TimeObject}->SystemTime2Date(
+                SystemTime => $PendingTime,
+                );
+
+            # set pending time
+            $Self->{TicketObject}->TicketPendingTimeSet(
+                Year     => $Year,
+                Month    => $Month,
+                Day      => $Day,
+                Hour     => $Hour,
+                Minute   => $Min,
+                TicketID => $Param{TicketID},
+                UserID   => $Param{UserID},
+            );
+        }
     }
 
     # set customer id and customer user
@@ -955,6 +1092,30 @@ sub _JobRunTicket {
         );
     }
 
+    # set new responsible
+    if ( $Param{Config}->{New}->{Responsible} ) {
+        if ( $Self->{NoticeSTDOUT} ) {
+            print
+                "  - set responsible of Ticket $Ticket to '$Param{Config}->{New}->{Responsible}'\n";
+        }
+        $Self->{TicketObject}->TicketResponsibleSet(
+            TicketID => $Param{TicketID},
+            UserID   => $Param{UserID},
+            NewUser  => $Param{Config}->{New}->{Responsible},
+        );
+    }
+    if ( $Param{Config}->{New}->{ResponsibleID} ) {
+        if ( $Self->{NoticeSTDOUT} ) {
+            print
+                "  - set responsible id of Ticket $Ticket to '$Param{Config}->{New}->{ResponsibleID}'\n";
+        }
+        $Self->{TicketObject}->TicketResponsibleSet(
+            TicketID  => $Param{TicketID},
+            UserID    => $Param{UserID},
+            NewUserID => $Param{Config}->{New}->{ResponsibleID},
+        );
+    }
+
     # set new lock
     if ( $Param{Config}->{New}->{Lock} ) {
         if ( $Self->{NoticeSTDOUT} ) {
@@ -977,49 +1138,44 @@ sub _JobRunTicket {
         );
     }
 
-    # set ticket free text options
-    for ( 1 .. 16 ) {
-        if (
-            defined $Param{Config}->{New}->{"TicketFreeKey$_"}
-            || defined $Param{Config}->{New}->{"TicketFreeText$_"}
-            )
-        {
-            my %Data;
-            $Data{TicketID} = $Param{TicketID};
-            $Data{UserID}   = $Param{UserID};
-            $Data{Counter}  = $_;
+    # set new dynamic fields options
+    # cycle trough the activated Dynamic Fields for this screen
+    DYNAMICFIELD:
+    for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
+        next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
 
-            if ( defined $Param{Config}->{New}->{"TicketFreeKey$_"} ) {
-                $Data{Key} = $Param{Config}->{New}->{"TicketFreeKey$_"};
-            }
+        # extract the dynamic field value form the web request
+        my $Value = $Self->{BackendObject}->EditFieldValueGet(
+            DynamicFieldConfig => $DynamicFieldConfig,
+            Template           => $Param{Config}->{New},
+            TransformDates     => 0,
+        );
 
-            # insert the freefieldkey, if only one key is possible
-            if (
-                !$Data{Key}
-                && ref $Self->{ConfigObject}->Get( 'TicketFreeKey' . $_ ) eq 'HASH'
-                )
-            {
-                my %TicketFreeKey = %{ $Self->{ConfigObject}->Get( 'TicketFreeKey' . $_ ) };
-                my @FreeKey       = keys %TicketFreeKey;
+        if ( defined $Value && $Value ne '' ) {
+            my $Success = $Self->{BackendObject}->ValueSet(
+                DynamicFieldConfig => $DynamicFieldConfig,
+                ObjectID           => $Param{TicketID},
+                Value              => $Value,
+                UserID             => 1,
+            );
 
-                if ( $#FreeKey == 0 ) {
-                    $Data{Key} = $TicketFreeKey{ $FreeKey[0] };
+            if ($Success) {
+                if ( $Self->{NoticeSTDOUT} ) {
+                    my $ValueStrg = $Self->{BackendObject}->ReadableValueRender(
+                        DynamicFieldConfig => $DynamicFieldConfig,
+                        Value              => $Value,
+                    );
+                    print "  - set ticket dynamic field $DynamicFieldConfig->{Name} "
+                        . "of Ticket $Ticket to $ValueStrg->{Title} '\n";
                 }
             }
-
-            if ( defined $Param{Config}->{New}->{"TicketFreeText$_"} ) {
-                $Data{Value} = $Param{Config}->{New}->{"TicketFreeText$_"};
+            else {
+                $Self->{LogObject}->Log(
+                    Priority => 'error',
+                    Message  => "Coud not set dynamic field $DynamicFieldConfig->{Name} "
+                        . "for Ticket $Ticket.",
+                );
             }
-
-            if ( $Self->{NoticeSTDOUT} ) {
-                if ( defined $Data{Key} ) {
-                    print "  - set ticket free text of Ticket $Ticket to Key: '$Data{Key}'\n";
-                }
-                if ( defined $Data{Value} ) {
-                    print "  - set ticket free text of Ticket $Ticket to Text: '$Data{Value}'\n";
-                }
-            }
-            $Self->{TicketObject}->TicketFreeTextSet(%Data);
         }
     }
 
@@ -1178,6 +1334,6 @@ did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =head1 VERSION
 
-$Revision: 1.70.2.2 $ $Date: 2012/02/15 22:45:18 $
+$Revision: 1.81 $ $Date: 2012-02-14 08:40:37 $
 
 =cut
