@@ -1,14 +1,15 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/System/Main.pm - main core components
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
+# --
+# $Id: Main.pm,v 1.63.2.2 2013-02-05 10:43:21 mg Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Main;
-## nofilter(TidyAll::Plugin::OTRS::Perl::Dumper)
-## nofilter(TidyAll::Plugin::OTRS::Perl::Require)
 
 use strict;
 use warnings;
@@ -17,33 +18,48 @@ use Digest::MD5 qw(md5_hex);
 use Data::Dumper;
 use File::stat;
 use Unicode::Normalize;
-use List::Util qw();
-use Fcntl qw(:flock);
-use Encode;
 
-use Kernel::System::VariableCheck qw(IsStringWithData);
+use Kernel::System::Encode;
 
-our @ObjectDependencies = (
-    'Kernel::System::Encode',
-    'Kernel::System::Log',
-    'Kernel::System::Storable',
-);
+use vars qw($VERSION);
+$VERSION = qw($Revision: 1.63.2.2 $) [1];
 
 =head1 NAME
 
 Kernel::System::Main - main object
 
-=head1 DESCRIPTION
+=head1 SYNOPSIS
 
 All main functions to load modules, die, and handle files.
 
 =head1 PUBLIC INTERFACE
 
-=head2 new()
+=over 4
 
-create new object. Do not use it directly, instead use:
+=cut
 
-    my $MainObject = $Kernel::OM->Get('Kernel::System::Main');
+=item new()
+
+create new object
+
+    use Kernel::Config;
+    use Kernel::System::Encode;
+    use Kernel::System::Log;
+    use Kernel::System::Main;
+
+    my $ConfigObject = Kernel::Config->new();
+    my $EncodeObject = Kernel::System::Encode->new(
+        ConfigObject => $ConfigObject,
+    );
+    my $LogObject = Kernel::System::Log->new(
+        ConfigObject => $ConfigObject,
+        EncodeObject => $EncodeObject,
+    );
+    my $MainObject = Kernel::System::Main->new(
+        ConfigObject => $ConfigObject,
+        EncodeObject => $EncodeObject,
+        LogObject    => $LogObject,
+    );
 
 =cut
 
@@ -54,87 +70,103 @@ sub new {
     my $Self = {};
     bless( $Self, $Type );
 
+    # check needed objects
+    for my $Object (qw(ConfigObject LogObject)) {
+        $Self->{$Object} = $Param{$Object} || die "Got no $Object!";
+    }
+
+    # get or create encode object
+    $Self->{EncodeObject} = $Param{EncodeObject};
+    $Self->{EncodeObject} ||= Kernel::System::Encode->new(%Param);
+
+    # set debug mode
+    $Self->{Debug} = $Param{Debug} || 0;
+
     return $Self;
 }
 
-=head2 Require()
+=item Require()
 
 require/load a module
 
-    my $Loaded = $MainObject->Require(
-        'Kernel::System::Example',
-        Silent => 1,                # optional, no log entry if module was not found
-    );
+    my $Loaded = $MainObject->Require('Kernel::System::Example');
 
 =cut
 
 sub Require {
-    my ( $Self, $Module, %Param ) = @_;
+    my ( $Self, $Module ) = @_;
 
     if ( !$Module ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+        $Self->{LogObject}->Log(
             Priority => 'error',
             Message  => 'Need module!',
         );
         return;
     }
 
-    eval {
-        my $FileName = $Module =~ s{::}{/}smxgr;
-        require $FileName . '.pm';
-    };
+    # prepare module
+    $Module =~ s/::/\//g;
+    $Module .= '.pm';
 
-    # Handle errors.
+    # just return if it's already loaded
+    return 1 if $INC{$Module};
+
+    my $Result;
+    my $File;
+
+    # find full path of module
+    PREFIX:
+    for my $Prefix (@INC) {
+        $File = $Prefix . '/' . $Module;
+
+        next PREFIX if !-f $File;
+
+        $Result = do $File;
+
+        last PREFIX;
+    }
+
+    # if there was an error
     if ($@) {
 
-        if ( !$Param{Silent} ) {
-            my $Message = $@;
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Caller   => 1,
-                Priority => 'error',
-                Message  => $Message,
-            );
-        }
+        # log error
+        $Self->{LogObject}->Log(
+            Caller   => 1,
+            Priority => 'error',
+            Message  => "$@",
+        );
 
         return;
     }
 
-    return 1;
-}
+    # return true if module is loaded
+    if ( !$Result ) {
 
-=head2 RequireBaseClass()
+        # if there is no file, show not found error
+        $Self->{LogObject}->Log(
+            Caller   => 1,
+            Priority => 'error',
+            Message  => "Module $Module not found!",
+        );
 
-require/load a module and add it as a base class to the
-calling package, if not already present (this check is needed
-for persistent environments).
-
-    my $Loaded = $MainObject->RequireBaseClass(
-        'Kernel::System::Example',
-    );
-
-=cut
-
-sub RequireBaseClass {
-    my ( $Self, $Module ) = @_;
-
-    # Load the module, if not already loaded.
-    return if !$Self->Require($Module);
-
-    no strict 'refs';    ## no critic
-    my $CallingClass = caller(0);
-
-    # Check if the base class was already loaded.
-    # This can happen in persistent environments as mod_perl (see bug#9686).
-    if ( List::Util::first { $_ eq $Module } @{"${CallingClass}::ISA"} ) {
-        return 1;    # nothing to do now
+        return;
     }
 
-    push @{"${CallingClass}::ISA"}, $Module;
+    # add module
+    $INC{$Module} = $File;
+
+    # log debug message
+    if ( $Self->{Debug} > 1 ) {
+        $Self->{LogObject}->Log(
+            Priority => 'debug',
+            Message  => "Module: $Module loaded!",
+        );
+    }
 
     return 1;
 }
 
-=head2 Die()
+=item Die()
 
 to die
 
@@ -148,7 +180,7 @@ sub Die {
     $Message = $Message || 'Died!';
 
     # log message
-    $Kernel::OM->Get('Kernel::System::Log')->Log(
+    $Self->{LogObject}->Log(
         Caller   => 1,
         Priority => 'error',
         Message  => $Message,
@@ -157,7 +189,7 @@ sub Die {
     exit;
 }
 
-=head2 FilenameCleanUp()
+=item FilenameCleanUp()
 
 to clean up filenames which can be used in any case (also quoting is done)
 
@@ -176,40 +208,24 @@ to clean up filenames which can be used in any case (also quoting is done)
 sub FilenameCleanUp {
     my ( $Self, %Param ) = @_;
 
-    if ( !IsStringWithData( $Param{Filename} ) ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+    if ( !$Param{Filename} ) {
+        $Self->{LogObject}->Log(
             Priority => 'error',
             Message  => 'Need Filename!',
         );
         return;
     }
 
-    # escape if cleanup is not needed
-    if ( $Param{NoFilenameClean} ) {
-        return $Param{Filename};
-    }
-
-    my $Type = lc( $Param{Type} || 'local' );
-
-    if ( $Type eq 'md5' ) {
-        $Kernel::OM->Get('Kernel::System::Encode')->EncodeOutput( \$Param{Filename} );
+    if ( $Param{Type} && $Param{Type} =~ /^md5/i ) {
+        $Self->{EncodeObject}->EncodeOutput( \$Param{Filename} );
         $Param{Filename} = md5_hex( $Param{Filename} );
     }
 
     # replace invalid token for attachment file names
-    elsif ( $Type eq 'attachment' ) {
+    elsif ( $Param{Type} && $Param{Type} =~ /^attachment/i ) {
 
-        # trim whitespace
-        $Param{Filename} =~ s/^\s+|\r|\n|\s+$//g;
-
-        # strip leading dots
-        $Param{Filename} =~ s/^\.+//;
-
-        # only whitelisted characters allowed in filename for security
-        $Param{Filename} =~ s/[^\w\-+.#_]/_/g;
-
-        # Enclosed alphanumerics are kept on older Perl versions, make sure to replace them too.
-        $Param{Filename} =~ s/[\x{2460}-\x{24FF}]/_/g;
+        # replace invalid token like < > ? " : ; | \ / or *
+        $Param{Filename} =~ s/[ <>\?":\\\*\|\/;\[\]]/_/g;
 
         # replace utf8 and iso
         $Param{Filename} =~ s/(\x{00C3}\x{00A4}|\x{00A4})/ae/g;
@@ -221,110 +237,49 @@ sub FilenameCleanUp {
         $Param{Filename} =~ s/(\x{00C3}\x{009F}|\x{00DF})/ss/g;
         $Param{Filename} =~ s/-+/-/g;
 
-        # separate filename and extension
-        my $FileName = $Param{Filename};
-        my $FileExt  = '';
-        if ( $Param{Filename} =~ /(.*)\.+([^.]+)$/ ) {
-            $FileName = $1;
-            $FileExt  = '.' . $2;
-        }
-
-        if ( length $FileName ) {
-            my $ModifiedName = $FileName . $FileExt;
-
-            while ( length encode( 'UTF-8', $ModifiedName ) > 220 ) {
-
-                # Remove character by character starting from the end of the filename string
-                #   until we get acceptable 220 byte long filename size including extension.
-                if ( length $FileName > 1 ) {
-                    chop $FileName;
-                }
-
-                # If we reached minimum filename length, remove characters from the end of the extension string.
-                else {
-                    chop $FileExt;
-                }
-
-                $ModifiedName = $FileName . $FileExt;
+        # cut the string if too long
+        if ( length( $Param{Filename} ) > 100 ) {
+            my $Ext = '';
+            if ( $Param{Filename} =~ /^.*(\.(...|....))$/ ) {
+                $Ext = $1;
             }
-            $Param{Filename} = $ModifiedName;
+            $Param{Filename} = substr( $Param{Filename}, 0, 95 ) . $Ext;
         }
     }
     else {
 
-        # trim whitespace
-        $Param{Filename} =~ s/^\s+|\r|\n|\s+$//g;
-
-        # strip leading dots
-        $Param{Filename} =~ s/^\.+//;
-
-        # only whitelisted characters allowed in filename for security
-        if ( !$Param{NoReplace} ) {
-            $Param{Filename} =~ s/[^\w\-+.#_]/_/g;
-
-            # Enclosed alphanumerics are kept on older Perl versions, make sure to replace them too.
-            $Param{Filename} =~ s/[\x{2460}-\x{24FF}]/_/g;
-        }
-
-        # separate filename and extension
-        my $FileName = $Param{Filename};
-        my $FileExt  = '';
-        if ( $Param{Filename} =~ /(.*)\.+([^.]+)$/ ) {
-            $FileName = $1;
-            $FileExt  = '.' . $2;
-        }
-
-        if ( length $FileName ) {
-            my $ModifiedName = $FileName . $FileExt;
-
-            while ( length encode( 'UTF-8', $ModifiedName ) > 220 ) {
-
-                # Remove character by character starting from the end of the filename string
-                #   until we get acceptable 220 byte long filename size including extension.
-                if ( length $FileName > 1 ) {
-                    chop $FileName;
-                }
-
-                # If we reached minimum filename length, remove characters from the end of the extension string.
-                else {
-                    chop $FileExt;
-                }
-
-                $ModifiedName = $FileName . $FileExt;
-            }
-
-            $Param{Filename} = $ModifiedName;
-        }
+        # replace invalid token like [ ] * : ? " < > ; | \ /
+        $Param{Filename} =~ s/[<>\?":\\\*\|\/;\[\]]/_/g;
     }
 
     return $Param{Filename};
 }
 
-=head2 FileRead()
+=item FileRead()
 
 to read files from file system
 
     my $ContentSCALARRef = $MainObject->FileRead(
-        Directory => 'c:\some\location',
-        Filename  => 'file2read.txt',
+        Directory => 'c:\some\location\me_to',
+        Filename  => 'alal.xml',
         # or Location
-        Location  => 'c:\some\location\file2read.txt',
+        Location  => 'c:\some\location\me_to\alal.xml'
     );
 
     my $ContentARRAYRef = $MainObject->FileRead(
-        Directory => 'c:\some\location',
-        Filename  => 'file2read.txt',
+        Directory => 'c:\some\location\me_to',
+        Filename  => 'alal.xml',
         # or Location
-        Location  => 'c:\some\location\file2read.txt',
+        Location  => 'c:\some\location\me_to\alal.xml'
 
         Result    => 'ARRAY', # optional - SCALAR|ARRAY
     );
 
     my $ContentSCALARRef = $MainObject->FileRead(
-        Directory       => 'c:\some\location',
-        Filename        => 'file2read.txt',
+        Directory       => 'c:\some\location\me_to',
+        Filename        => 'alal.xml',
         # or Location
-        Location        => 'c:\some\location\file2read.txt',
+        Location        => 'c:\some\location\me_to\alal.xml'
 
         Mode            => 'binmode', # optional - binmode|utf8
         Type            => 'Local',   # optional - Local|Attachment|MD5
@@ -343,7 +298,7 @@ sub FileRead {
         # filename clean up
         $Param{Filename} = $Self->FilenameCleanUp(
             Filename => $Param{Filename},
-            Type     => $Param{Type} || 'Local',    # Local|Attachment|MD5
+            Type => $Param{Type} || 'Local',    # Local|Attachment|MD5
         );
         $Param{Location} = "$Param{Directory}/$Param{Filename}";
     }
@@ -353,13 +308,24 @@ sub FileRead {
         $Param{Location} =~ s{//}{/}xmsg;
     }
     else {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+        $Self->{LogObject}->Log(
             Priority => 'error',
             Message  => 'Need Filename and Directory or Location!',
         );
 
     }
 
+    # check if file exists
+    if ( !-e $Param{Location} ) {
+        if ( !$Param{DisableWarnings} ) {
+            $Self->{LogObject}->Log(
+                Priority => 'error',
+                Message  => "File '$Param{Location}' doesn't exist!"
+            );
+        }
+        return;
+    }
+
     # set open mode
     my $Mode = '<';
     if ( $Param{Mode} && $Param{Mode} =~ m{ \A utf-?8 \z }xmsi ) {
@@ -367,32 +333,20 @@ sub FileRead {
     }
 
     # return if file can not open
-    if ( !open $FH, $Mode, $Param{Location} ) {    ## no critic
-        my $Error = $!;
-
+    if ( !open $FH, $Mode, $Param{Location} ) {
         if ( !$Param{DisableWarnings} ) {
-
-            # Check if file exists only if system was not able to open it (to get better error message).
-            if ( !-e $Param{Location} ) {
-                $Kernel::OM->Get('Kernel::System::Log')->Log(
-                    Priority => 'error',
-                    Message  => "File '$Param{Location}' doesn't exist!",
-                );
-            }
-            else {
-                $Kernel::OM->Get('Kernel::System::Log')->Log(
-                    Priority => 'error',
-                    Message  => "Can't open '$Param{Location}': $Error",
-                );
-            }
+            $Self->{LogObject}->Log(
+                Priority => 'error',
+                Message  => "Can't open '$Param{Location}': $!",
+            );
         }
         return;
     }
 
     # lock file (Shared Lock)
-    if ( !flock $FH, LOCK_SH ) {
+    if ( !flock $FH, 1 ) {
         if ( !$Param{DisableWarnings} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
+            $Self->{LogObject}->Log(
                 Priority => 'error',
                 Message  => "Can't lock '$Param{Location}': $!",
             );
@@ -421,34 +375,31 @@ sub FileRead {
     return \$String;
 }
 
-=head2 FileWrite()
+=item FileWrite()
 
 to write data to file system
 
     my $FileLocation = $MainObject->FileWrite(
         Directory => 'c:\some\location',
-        Filename  => 'file2write.txt',
+        Filename  => 'me_to/alal.xml',
         # or Location
-        Location  => 'c:\some\location\file2write.txt',
+        Location  => 'c:\some\location\me_to\alal.xml'
 
         Content   => \$Content,
     );
 
     my $FileLocation = $MainObject->FileWrite(
         Directory  => 'c:\some\location',
-        Filename   => 'file2write.txt',
+        Filename   => 'me_to/alal.xml',
         # or Location
-        Location   => 'c:\some\location\file2write.txt',
+        Location   => 'c:\some\location\me_to\alal.xml'
 
         Content    => \$Content,
         Mode       => 'binmode', # binmode|utf8
         Type       => 'Local',   # optional - Local|Attachment|MD5
-        Permission => '644',     # optional - unix file permissions
+        Permission => '644',     # unix file permissions
     );
 
-Platform note: MacOS (HFS+) stores filenames as Unicode C<NFD> internally,
-and DirectoryRead() will also report them as C<NFD>.
-
 =cut
 
 sub FileWrite {
@@ -458,10 +409,8 @@ sub FileWrite {
 
         # filename clean up
         $Param{Filename} = $Self->FilenameCleanUp(
-            Filename        => $Param{Filename},
-            Type            => $Param{Type} || 'Local',    # Local|Attachment|MD5
-            NoFilenameClean => $Param{NoFilenameClean},
-            NoReplace       => $Param{NoReplace},
+            Filename => $Param{Filename},
+            Type => $Param{Type} || 'Local',    # Local|Attachment|MD5
         );
         $Param{Location} = "$Param{Directory}/$Param{Filename}";
     }
@@ -471,7 +420,7 @@ sub FileWrite {
         $Param{Location} =~ s/\/\//\//g;
     }
     else {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+        $Self->{LogObject}->Log(
             Priority => 'error',
             Message  => 'Need Filename and Directory or Location!',
         );
@@ -495,8 +444,8 @@ sub FileWrite {
 
     # return if file can not open
     my $FH;
-    if ( !open $FH, $Mode, $Param{Location} ) {    ## no critic
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+    if ( !open $FH, $Mode, $Param{Location} ) {
+        $Self->{LogObject}->Log(
             Priority => 'error',
             Message  => "Can't write '$Param{Location}': $!",
         );
@@ -504,8 +453,8 @@ sub FileWrite {
     }
 
     # lock file (Exclusive Lock)
-    if ( !flock $FH, LOCK_EX ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+    if ( !flock $FH, 2 ) {
+        $Self->{LogObject}->Log(
             Priority => 'error',
             Message  => "Can't lock '$Param{Location}': $!",
         );
@@ -518,7 +467,7 @@ sub FileWrite {
     if ( !$Param{Mode} || lc $Param{Mode} eq 'binmode' ) {
 
         # make sure, that no utf8 stamp exists (otherway perl will do auto convert to iso)
-        $Kernel::OM->Get('Kernel::System::Encode')->EncodeOutput( $Param{Content} );
+        $Self->{EncodeObject}->EncodeOutput( $Param{Content} );
 
         # set file handle to binmode
         binmode $FH;
@@ -549,7 +498,7 @@ sub FileWrite {
     return $Param{Location};
 }
 
-=head2 FileDelete()
+=item FileDelete()
 
 to delete a file from file system
 
@@ -572,9 +521,8 @@ sub FileDelete {
 
         # filename clean up
         $Param{Filename} = $Self->FilenameCleanUp(
-            Filename  => $Param{Filename},
-            Type      => $Param{Type} || 'Local',    # Local|Attachment|MD5
-            NoReplace => $Param{NoReplace},
+            Filename => $Param{Filename},
+            Type => $Param{Type} || 'Local',    # Local|Attachment|MD5
         );
         $Param{Location} = "$Param{Directory}/$Param{Filename}";
     }
@@ -584,40 +532,38 @@ sub FileDelete {
         $Param{Location} =~ s/\/\//\//g;
     }
     else {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+        $Self->{LogObject}->Log(
             Priority => 'error',
             Message  => 'Need Filename and Directory or Location!',
         );
     }
 
-    # try to delete file
-    if ( !unlink( $Param{Location} ) ) {
-        my $Error = $!;
-
+    # check if file exists
+    if ( !-e $Param{Location} ) {
         if ( !$Param{DisableWarnings} ) {
-
-            # Check if file exists only in case that delete failed.
-            if ( !-e $Param{Location} ) {
-                $Kernel::OM->Get('Kernel::System::Log')->Log(
-                    Priority => 'error',
-                    Message  => "File '$Param{Location}' doesn't exist!",
-                );
-            }
-            else {
-                $Kernel::OM->Get('Kernel::System::Log')->Log(
-                    Priority => 'error',
-                    Message  => "Can't delete '$Param{Location}': $Error",
-                );
-            }
+            $Self->{LogObject}->Log(
+                Priority => 'error',
+                Message  => "File '$Param{Location}' doesn't exist!"
+            );
         }
+        return;
+    }
 
+    # delete file
+    if ( !unlink( $Param{Location} ) ) {
+        if ( !$Param{DisableWarnings} ) {
+            $Self->{LogObject}->Log(
+                Priority => 'error',
+                Message  => "Can't delete '$Param{Location}': $!",
+            );
+        }
         return;
     }
 
     return 1;
 }
 
-=head2 FileGetMTime()
+=item FileGetMTime()
 
 get timestamp of file change time
 
@@ -639,7 +585,7 @@ sub FileGetMTime {
         # filename clean up
         $Param{Filename} = $Self->FilenameCleanUp(
             Filename => $Param{Filename},
-            Type     => $Param{Type} || 'Local',    # Local|Attachment|MD5
+            Type => $Param{Type} || 'Local',    # Local|Attachment|MD5
         );
         $Param{Location} = "$Param{Directory}/$Param{Filename}";
     }
@@ -649,44 +595,41 @@ sub FileGetMTime {
         $Param{Location} =~ s{//}{/}xmsg;
     }
     else {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+        $Self->{LogObject}->Log(
             Priority => 'error',
             Message  => 'Need Filename and Directory or Location!',
         );
 
     }
 
-    # get file metadata
-    my $Stat = stat( $Param{Location} );
-
-    if ( !$Stat ) {
-        my $Error = $!;
-
+    # check if file exists
+    if ( !-e $Param{Location} ) {
         if ( !$Param{DisableWarnings} ) {
-
-            # Check if file exists only if system was not able to open it (to get better error message).
-            if ( !-e $Param{Location} ) {
-                $Kernel::OM->Get('Kernel::System::Log')->Log(
-                    Priority => 'error',
-                    Message  => "File '$Param{Location}' doesn't exist!"
-                );
-            }
-            else {
-                $Kernel::OM->Get('Kernel::System::Log')->Log(
-                    Priority => 'error',
-                    Message  => "Cannot stat file '$Param{Location}': $Error",
-                );
-            }
+            $Self->{LogObject}->Log(
+                Priority => 'error',
+                Message  => "File '$Param{Location}' doesn't exist!"
+            );
         }
         return;
     }
 
-    return $Stat->mtime();
+    # get file metadata
+    my $Stat = stat( $Param{Location} );
+
+    if ( !$Stat ) {
+        $Self->{LogObject}->Log(
+            Priority => 'error',
+            Message  => "Cannot stat file '$Param{Location}': $!"
+        );
+        return;
+    }
+
+    return $Stat->mtime;
 }
 
-=head2 MD5sum()
+=item MD5sum()
 
-get an C<MD5> sum of a file or a string
+get a md5 sum of a file or a string
 
     my $MD5Sum = $MainObject->MD5sum(
         Filename => '/path/to/me_to_alal.xml',
@@ -706,35 +649,33 @@ get an C<MD5> sum of a file or a string
 sub MD5sum {
     my ( $Self, %Param ) = @_;
 
-    if ( !$Param{Filename} && !defined( $Param{String} ) ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+    if ( !$Param{Filename} && !$Param{String} ) {
+        $Self->{LogObject}->Log(
             Priority => 'error',
             Message  => 'Need Filename or String!',
         );
         return;
     }
 
+    # check if file exists
+    if ( $Param{Filename} && !-e $Param{Filename} ) {
+        $Self->{LogObject}->Log(
+            Priority => 'error',
+            Message  => "File '$Param{Filename}' doesn't exist!",
+        );
+        return;
+    }
+
     # md5sum file
     if ( $Param{Filename} ) {
 
         # open file
         my $FH;
-        if ( !open $FH, '<', $Param{Filename} ) {    ## no critic
-            my $Error = $!;
-
-            # Check if file exists only if system was not able to open it (to get better error message).
-            if ( !-e $Param{Filename} ) {
-                $Kernel::OM->Get('Kernel::System::Log')->Log(
-                    Priority => 'error',
-                    Message  => "File '$Param{Filename}' doesn't exist!",
-                );
-            }
-            else {
-                $Kernel::OM->Get('Kernel::System::Log')->Log(
-                    Priority => 'error',
-                    Message  => "Can't read '$Param{Filename}': $Error",
-                );
-            }
+        if ( !open $FH, '<', $Param{Filename} ) {
+            $Self->{LogObject}->Log(
+                Priority => 'error',
+                Message  => "Can't read '$Param{Filename}': $!",
+            );
             return;
         }
 
@@ -745,22 +686,19 @@ sub MD5sum {
         return $MD5sum;
     }
 
-    # get encode object
-    my $EncodeObject = $Kernel::OM->Get('Kernel::System::Encode');
-
     # md5sum string
     if ( !ref $Param{String} ) {
-        $EncodeObject->EncodeOutput( \$Param{String} );
+        $Self->{EncodeObject}->EncodeOutput( \$Param{String} );
         return md5_hex( $Param{String} );
     }
 
     # md5sum scalar reference
     if ( ref $Param{String} eq 'SCALAR' ) {
-        $EncodeObject->EncodeOutput( $Param{String} );
+        $Self->{EncodeObject}->EncodeOutput( $Param{String} );
         return md5_hex( ${ $Param{String} } );
     }
 
-    $Kernel::OM->Get('Kernel::System::Log')->Log(
+    $Self->{LogObject}->Log(
         Priority => 'error',
         Message  => "Need a SCALAR reference like 'String => \$Content' in String param.",
     );
@@ -768,7 +706,7 @@ sub MD5sum {
     return;
 }
 
-=head2 Dump()
+=item Dump()
 
 dump variable to an string
 
@@ -796,10 +734,7 @@ sub Dump {
 
     # check needed data
     if ( !defined $Data ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => "Need \$String in Dump()!"
-        );
+        $Self->{LogObject}->Log( Priority => 'error', Message => "Need \$String in Dump()!" );
         return;
     }
 
@@ -808,10 +743,7 @@ sub Dump {
         $Type = 'binary';
     }
     if ( $Type ne 'ascii' && $Type ne 'binary' ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => "Invalid Type '$Type'!"
-        );
+        $Self->{LogObject}->Log( Priority => 'error', Message => "Invalid Type '$Type'!" );
         return;
     }
 
@@ -821,23 +753,23 @@ sub Dump {
     # sort hash keys
     $Data::Dumper::Sortkeys = 1;
 
-    # This Dump() is using Data::Dumper with a utf8 workarounds to handle
+    # This Dump() is using Data::Dumer with a utf8 workarounds to handle
     # the bug [rt.cpan.org #28607] Data::Dumper::Dumper is dumping utf8
     # strings as latin1/8bit instead of utf8. Use Storable module used for
     # workaround.
     # -> http://rt.cpan.org/Ticket/Display.html?id=28607
-    if ( $Type eq 'binary' ) {
+    if ( $Self->Require('Storable') && $Type eq 'binary' ) {
 
         # Clone the data because we need to disable the utf8 flag in all
-        # reference variables and do not to want to do this in the orig.
-        # variables because they will still used in the system.
-        my $DataNew = $Kernel::OM->Get('Kernel::System::Storable')->Clone( Data => \$Data );
+        # reference variables and we want not to do this in the orig.
+        # variables because this will still used in the system.
+        my $DataNew = Storable::dclone( \$Data );
 
         # Disable utf8 flag.
         $Self->_Dump($DataNew);
 
         # Dump it as binary strings.
-        my $String = Data::Dumper::Dumper( ${$DataNew} );    ## no critic
+        my $String = Data::Dumper::Dumper( ${$DataNew} );
 
         # Enable utf8 flag.
         Encode::_utf8_on($String);
@@ -846,232 +778,10 @@ sub Dump {
     }
 
     # fallback if Storable can not be loaded
-    return Data::Dumper::Dumper($Data);                      ## no critic
+    return Data::Dumper::Dumper($Data);
 
 }
 
-=head2 DirectoryRead()
-
-reads a directory and returns an array with results.
-
-    my @FilesInDirectory = $MainObject->DirectoryRead(
-        Directory => '/tmp',
-        Filter    => 'Filenam*',
-    );
-
-    my @FilesInDirectory = $MainObject->DirectoryRead(
-        Directory => $Path,
-        Filter    => '*',
-    );
-
-read all files in subdirectories as well (recursive):
-
-    my @FilesInDirectory = $MainObject->DirectoryRead(
-        Directory => $Path,
-        Filter    => '*',
-        Recursive => 1,
-    );
-
-You can pass several additional filters at once:
-
-    my @FilesInDirectory = $MainObject->DirectoryRead(
-        Directory => '/tmp',
-        Filter    => \@MyFilters,
-    );
-
-The result strings are absolute paths, and they are converted to utf8.
-
-Use the 'Silent' parameter to suppress log messages when a directory
-does not have to exist:
-
-    my @FilesInDirectory = $MainObject->DirectoryRead(
-        Directory => '/special/optional/directory/',
-        Filter    => '*',
-        Silent    => 1,     # will not log errors if the directory does not exist
-    );
-
-Platform note: MacOS (HFS+) stores filenames as Unicode C<NFD> internally,
-and DirectoryRead() will also report them as C<NFD>.
-
-=cut
-
-sub DirectoryRead {
-    my ( $Self, %Param ) = @_;
-
-    # check needed params
-    for my $Needed (qw(Directory Filter)) {
-        if ( !$Param{$Needed} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Message  => "Needed $Needed: $!",
-                Priority => 'error',
-            );
-            return;
-        }
-    }
-
-    # if directory doesn't exists stop
-    if ( !-d $Param{Directory} && !$Param{Silent} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Message  => "Directory doesn't exist: $Param{Directory}: $!",
-            Priority => 'error',
-        );
-        return;
-    }
-
-    # check Filter param
-    if ( ref $Param{Filter} ne '' && ref $Param{Filter} ne 'ARRAY' ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Message  => 'Filter param need to be scalar or array ref!',
-            Priority => 'error',
-        );
-        return;
-    }
-
-    # prepare non array filter
-    if ( ref $Param{Filter} ne 'ARRAY' ) {
-        $Param{Filter} = [ $Param{Filter} ];
-    }
-
-    # executes glob for every filter
-    my @GlobResults;
-    my %Seen;
-
-    for my $Filter ( @{ $Param{Filter} } ) {
-        my @Glob = glob "$Param{Directory}/$Filter";
-
-        # look for repeated values
-        NAME:
-        for my $GlobName (@Glob) {
-
-            next NAME if !-e $GlobName;
-            if ( !$Seen{$GlobName} ) {
-                push @GlobResults, $GlobName;
-                $Seen{$GlobName} = 1;
-            }
-        }
-    }
-
-    if ( $Param{Recursive} ) {
-
-        # loop protection to prevent symlinks causing lockups
-        $Param{LoopProtection}++;
-        return if $Param{LoopProtection} > 100;
-
-        # check all files in current directory
-        my @Directories = glob "$Param{Directory}/*";
-
-        DIRECTORY:
-        for my $Directory (@Directories) {
-
-            # return if file is not a directory
-            next DIRECTORY if !-d $Directory;
-
-            # repeat same glob for directory
-            my @SubResult = $Self->DirectoryRead(
-                %Param,
-                Directory => $Directory,
-            );
-
-            # add result to hash
-            for my $Result (@SubResult) {
-                if ( !$Seen{$Result} ) {
-                    push @GlobResults, $Result;
-                    $Seen{$Result} = 1;
-                }
-            }
-        }
-    }
-
-    # if no results
-    return if !@GlobResults;
-
-    # get encode object
-    my $EncodeObject = $Kernel::OM->Get('Kernel::System::Encode');
-
-    # compose normalize every name in the file list
-    my @Results;
-    for my $Filename (@GlobResults) {
-
-        # First convert filename to utf-8, with additional Check parameter
-        # to replace possible malformed characters and prevent further errors.
-        $Filename = $EncodeObject->Convert2CharsetInternal(
-            Text  => $Filename,
-            From  => 'utf-8',
-            Check => 1,
-        );
-
-        push @Results, $Filename;
-    }
-
-    # always sort the result
-    @Results = sort @Results;
-
-    return @Results;
-}
-
-=head2 GenerateRandomString()
-
-generate a random string of defined length, and of a defined alphabet.
-defaults to a length of 16 and alphanumerics ( 0..9, A-Z and a-z).
-
-    my $String = $MainObject->GenerateRandomString();
-
-returns
-
-    $String = 'mHLOx7psWjMe5Pj7';
-
-with specific length:
-
-    my $String = $MainObject->GenerateRandomString(
-        Length => 32,
-    );
-
-returns
-
-    $String = 'azzHab72wIlAXDrxHexsI5aENsESxAO7';
-
-with specific length and alphabet:
-
-    my $String = $MainObject->GenerateRandomString(
-        Length     => 32,
-        Dictionary => [ 0..9, 'a'..'f' ], # hexadecimal
-        );
-
-returns
-
-    $String = '9fec63d37078fe72f5798d2084fea8ad';
-
-
-=cut
-
-sub GenerateRandomString {
-    my ( $Self, %Param ) = @_;
-
-    my $Length = $Param{Length} || 16;
-
-    # The standard list of characters in the dictionary. Don't use special chars here.
-    my @DictionaryChars = ( 0 .. 9, 'A' .. 'Z', 'a' .. 'z' );
-
-    # override dictionary with custom list if given
-    if ( $Param{Dictionary} && ref $Param{Dictionary} eq 'ARRAY' ) {
-        @DictionaryChars = @{ $Param{Dictionary} };
-    }
-
-    my $DictionaryLength = scalar @DictionaryChars;
-
-    # generate the string
-    my $String;
-
-    for ( 1 .. $Length ) {
-
-        my $Key = int rand $DictionaryLength;
-
-        $String .= $DictionaryChars[$Key];
-    }
-
-    return $String;
-}
-
 =begin Internal:
 
 =cut
@@ -1098,21 +808,11 @@ sub _Dump {
     # data is a hash reference
     if ( ref ${$Data} eq 'HASH' ) {
         KEY:
-        for my $Key ( sort keys %{ ${$Data} } ) {
+        for my $Key ( keys %{ ${$Data} } ) {
             next KEY if !defined ${$Data}->{$Key};
 
             # start recursion
             $Self->_Dump( \${$Data}->{$Key} );
-
-            my $KeyNew = $Key;
-
-            $Self->_Dump( \$KeyNew );
-
-            if ( $Key ne $KeyNew ) {
-
-                ${$Data}->{$KeyNew} = ${$Data}->{$Key};
-                delete ${$Data}->{$Key};
-            }
         }
 
         return;
@@ -1140,16 +840,7 @@ sub _Dump {
         return;
     }
 
-    # data is a JSON::PP::Boolean
-    if ( ref ${$Data} eq 'JSON::PP::Boolean' ) {
-
-        # start recursion
-        $Self->_Dump( ${$Data} );
-
-        return;
-    }
-
-    $Kernel::OM->Get('Kernel::System::Log')->Log(
+    $Self->{LogObject}->Log(
         Priority => 'error',
         Message  => "Unknown ref '" . ref( ${$Data} ) . "'!",
     );
@@ -1157,16 +848,134 @@ sub _Dump {
     return;
 }
 
+=item DirectoryRead()
+
+reads a directory and returns an array with results.
+
+    my @FilesInDirectory = $MainObject->DirectoryRead(
+        Directory => '/tmp',
+        Filter    => 'Filenam*',
+    );
+
+    my @FilesInDirectory = $MainObject->DirectoryRead(
+        Directory => $Path,
+        Filter    => '*',
+    );
+
+You can pass several additional filters at once:
+
+    my @FilesInDirectory = $MainObject->DirectoryRead(
+        Directory => '/tmp',
+        Filter    => \@MyFilters,
+    );
+
+The result strings are absolute paths, and they are converted to the
+internally used charset utf-8, if it is configured.
+
+Use the 'Silent' parameter to suppress log messages when a directory
+does not have to exist:
+
+    my @FilesInDirectory = $MainObject->DirectoryRead(
+        Directory => '/special/optional/directory/',
+        Filter    => '*',
+        Silent    => 1,     # will not log errors if the directory does not exist
+    );
+
+=cut
+
+sub DirectoryRead {
+    my ( $Self, %Param ) = @_;
+
+    # check needed params
+    for my $Needed (qw(Directory Filter)) {
+        if ( !$Param{$Needed} ) {
+            $Self->{LogObject}->Log(
+                Message  => "Needed $Needed: $!",
+                Priority => 'error',
+            );
+            return;
+        }
+    }
+
+    # if directory doesn't exists stop
+    if ( !-d $Param{Directory} && !$Param{Silent} ) {
+        $Self->{LogObject}->Log(
+            Message  => "Directory doesn't exist: $Param{Directory}: $!",
+            Priority => 'error',
+        );
+        return;
+    }
+
+    # check Filter param
+    if ( ref $Param{Filter} ne '' && ref $Param{Filter} ne 'ARRAY' ) {
+        $Self->{LogObject}->Log(
+            Message  => 'Filter param need to be scalar or array ref!',
+            Priority => 'error',
+        );
+        return;
+    }
+
+    # prepare non array filter
+    if ( ref $Param{Filter} ne 'ARRAY' ) {
+        $Param{Filter} = [ $Param{Filter} ];
+    }
+
+    # executes glob for every filter
+    my @GlobResults;
+    my %Seen;
+
+    for my $Filter ( @{ $Param{Filter} } ) {
+        my @Glob = glob "$Param{Directory}/$Filter";
+
+        # look for repeated values
+        for my $GlobName (@Glob) {
+            next if !-e $GlobName;
+            if ( !$Seen{$GlobName} ) {
+                push @GlobResults, $GlobName;
+                $Seen{$GlobName} = 1;
+            }
+        }
+    }
+
+    # if clean results
+    return if !@GlobResults;
+
+    # compose normalize every name in the file list
+    my @Results;
+    for my $Filename (@GlobResults) {
+
+        # first convert filename to utf-8 if utf-8 is used internally
+        $Filename = $Self->{EncodeObject}->Convert2CharsetInternal(
+            Text => $Filename,
+            From => 'utf-8',
+        );
+
+        push @Results, $Filename;
+    }
+
+    # always sort the result
+    return sort @Results;
+
+}
+
 1;
 
 =end Internal:
 
+=back
+
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
+
+=cut
+
+=head1 VERSION
+
+$Revision: 1.63.2.2 $ $Date: 2013-02-05 10:43:21 $
 
 =cut
