@@ -1,8 +1,8 @@
 # --
 # Kernel/System/Ticket/Event/Test.pm - test event module
-# Copyright (C) 2001-2010 OTRS AG, http://otrs.org/
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
-# $Id: Test.pm,v 1.11 2010-05-19 07:08:18 mb Exp $
+# $Id: Test.pm,v 1.12 2011-11-25 10:14:18 mg Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -15,7 +15,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION);
-$VERSION = qw($Revision: 1.11 $) [1];
+$VERSION = qw($Revision: 1.12 $) [1];
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -50,7 +50,10 @@ sub Run {
     }
 
     if ( $Param{Event} eq 'TicketCreate' ) {
-        my %Ticket = $Self->{TicketObject}->TicketGet( TicketID => $Param{Data}->{TicketID} );
+        my %Ticket = $Self->{TicketObject}->TicketGet(
+            TicketID      => $Param{Data}->{TicketID},
+            DynamicFields => 0,
+        );
         if ( $Ticket{State} eq 'Test' ) {
 
             # do some stuff
@@ -63,7 +66,10 @@ sub Run {
         }
     }
     elsif ( $Param{Event} eq 'TicketQueueUpdate' ) {
-        my %Ticket = $Self->{TicketObject}->TicketGet( TicketID => $Param{Data}->{TicketID} );
+        my %Ticket = $Self->{TicketObject}->TicketGet(
+            TicketID      => $Param{Data}->{TicketID},
+            DynamicFields => 0,
+        );
         if ( $Ticket{Queue} eq 'Test' ) {
 
             # do some stuff
