@@ -1,9 +1,12 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/System/Ticket/Event/TriggerEscalationStopEvents.pm - trigger escalation stop events
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
+# --
+# $Id: TriggerEscalationStopEvents.pm,v 1.2.2.1 2012-09-27 14:14:34 mb Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Ticket::Event::TriggerEscalationStopEvents;
@@ -11,10 +14,8 @@ package Kernel::System::Ticket::Event::TriggerEscalationStopEvents;
 use strict;
 use warnings;
 
-our @ObjectDependencies = (
-    'Kernel::System::Log',
-    'Kernel::System::Ticket',
-);
+use vars qw($VERSION);
+$VERSION = qw($Revision: 1.2.2.1 $) [1];
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -23,6 +24,14 @@ sub new {
     my $Self = {};
     bless( $Self, $Type );
 
+    # get needed objects
+    for my $Object (
+        qw(ConfigObject TicketObject LogObject UserObject CustomerUserObject TimeObject)
+        )
+    {
+        $Self->{$Object} = $Param{$Object} || die "Got no $Object!";
+    }
+
     return $Self;
 }
 
@@ -32,28 +41,19 @@ sub Run {
     # check needed stuff
     for my $Needed (qw(Event UserID Config)) {
         if ( !$Param{$Needed} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Priority => 'error',
-                Message  => "Need $Needed!"
-            );
+            $Self->{LogObject}->Log( Priority => 'error', Message => "Need $Needed!" );
             return;
         }
     }
     for my $Needed (qw(OldTicketData TicketID)) {
         if ( !$Param{Data}->{$Needed} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Priority => 'error',
-                Message  => "Need $Needed in Data!"
-            );
+            $Self->{LogObject}->Log( Priority => 'error', Message => "Need $Needed in Data!" );
             return;
         }
     }
 
-    # get ticket object
-    my $TicketObject = $Kernel::OM->Get('Kernel::System::Ticket');
-
     # get the current escalation status
-    my %Ticket = $TicketObject->TicketGet(
+    my %Ticket = $Self->{TicketObject}->TicketGet(
         TicketID      => $Param{Data}->{TicketID},
         UserID        => $Param{UserID},
         DynamicFields => 0,
@@ -65,13 +65,11 @@ sub Run {
         UpdateTimeEscalation        => 'EscalationUpdateTimeStop',
         SolutionTimeEscalation      => 'EscalationSolutionTimeStop',
     );
-
     while ( my ( $Attr, $Event ) = each %Attr2Event ) {
-
         if ( $Param{Data}->{OldTicketData}->{$Attr} && !$Ticket{$Attr} ) {
 
             # trigger the event
-            $TicketObject->EventHandler(
+            $Self->{TicketObject}->EventHandler(
                 Event  => $Event,
                 UserID => $Param{UserID},
                 Data   => {
@@ -81,7 +79,7 @@ sub Run {
             );
 
             # log the triggered event in the history
-            $TicketObject->HistoryAdd(
+            $Self->{TicketObject}->HistoryAdd(
                 TicketID     => $Param{Data}->{TicketID},
                 HistoryType  => $Event,
                 Name         => "%%$Event%%triggered",
