@@ -1,6 +1,9 @@
-#!/usr/bin/perl
+#!/usr/bin/perl -w
 # --
-# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
+# bin/otrs.GetTicketThread.pl - to print the whole ticket thread to STDOUT
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
+# --
+# $Id: otrs.GetTicketThread.pl,v 1.5.2.1 2012-08-01 23:16:27 mh Exp $
 # --
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU AFFERO General Public License as published by
@@ -9,12 +12,12 @@
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 # or see http://www.gnu.org/licenses/agpl.txt.
 # --
 
@@ -27,6 +30,9 @@ use lib dirname($RealBin);
 use lib dirname($RealBin) . '/Kernel/cpan-lib';
 use lib dirname($RealBin) . '/Custom';
 
+use vars qw($VERSION);
+$VERSION = qw($Revision: 1.5.2.1 $) [1];
+
 use Getopt::Std;
 
 use Kernel::Config;
@@ -42,8 +48,8 @@ my %Opts;
 getopt( 'htl', \%Opts );
 if ( $Opts{h} || !$Opts{t} ) {
     print
-        "otrs.GetTicketThread.pl - Prints out a ticket with all its articles.\n";
-    print "Copyright (C) 2001-2017 OTRS AG, http://otrs.com/\n";
+        "otrs.GetTicketThread.pl <Revision $VERSION> - Prints out a ticket with all its articles.\n";
+    print "Copyright (C) 2001-2014 OTRS AG, http://otrs.com/\n";
     print "usage: otrs.GetTicketThread.pl -t <TicketID> [-l article limit]\n";
     exit 1;
 }
