@@ -1,5 +1,5 @@
 # ----------------------------------------------------------
-#  driver: mysql, generated: 2010-08-03 10:09:04
+#  driver: mysql, generated: 2013-12-03 14:31:50
 # ----------------------------------------------------------
 # ----------------------------------------------------------
 #  create table valid
@@ -347,44 +347,6 @@ CREATE TABLE ticket (
     escalation_update_time INTEGER NOT NULL,
     escalation_response_time INTEGER NOT NULL,
     escalation_solution_time INTEGER NOT NULL,
-    freekey1 VARCHAR (80) NULL,
-    freetext1 VARCHAR (150) NULL,
-    freekey2 VARCHAR (80) NULL,
-    freetext2 VARCHAR (150) NULL,
-    freekey3 VARCHAR (80) NULL,
-    freetext3 VARCHAR (150) NULL,
-    freekey4 VARCHAR (80) NULL,
-    freetext4 VARCHAR (150) NULL,
-    freekey5 VARCHAR (80) NULL,
-    freetext5 VARCHAR (150) NULL,
-    freekey6 VARCHAR (80) NULL,
-    freetext6 VARCHAR (150) NULL,
-    freekey7 VARCHAR (80) NULL,
-    freetext7 VARCHAR (150) NULL,
-    freekey8 VARCHAR (80) NULL,
-    freetext8 VARCHAR (150) NULL,
-    freekey9 VARCHAR (80) NULL,
-    freetext9 VARCHAR (150) NULL,
-    freekey10 VARCHAR (80) NULL,
-    freetext10 VARCHAR (150) NULL,
-    freekey11 VARCHAR (80) NULL,
-    freetext11 VARCHAR (150) NULL,
-    freekey12 VARCHAR (80) NULL,
-    freetext12 VARCHAR (150) NULL,
-    freekey13 VARCHAR (80) NULL,
-    freetext13 VARCHAR (150) NULL,
-    freekey14 VARCHAR (80) NULL,
-    freetext14 VARCHAR (150) NULL,
-    freekey15 VARCHAR (80) NULL,
-    freetext15 VARCHAR (150) NULL,
-    freekey16 VARCHAR (80) NULL,
-    freetext16 VARCHAR (150) NULL,
-    freetime1 DATETIME NULL,
-    freetime2 DATETIME NULL,
-    freetime3 DATETIME NULL,
-    freetime4 DATETIME NULL,
-    freetime5 DATETIME NULL,
-    freetime6 DATETIME NULL,
     valid_id SMALLINT NOT NULL,
     archive_flag SMALLINT NOT NULL DEFAULT 0,
     create_time_unix BIGINT NOT NULL,
@@ -425,6 +387,7 @@ CREATE TABLE ticket_flag (
     ticket_value VARCHAR (50) NULL,
     create_time DATETIME NOT NULL,
     create_by INTEGER NOT NULL,
+    UNIQUE INDEX ticket_flag_per_user (ticket_id, ticket_key, create_by),
     INDEX ticket_flag_ticket_id (ticket_id),
     INDEX ticket_flag_ticket_id_create_by (ticket_id, create_by),
     INDEX ticket_flag_ticket_id_ticket_key (ticket_id, ticket_key)
@@ -492,10 +455,10 @@ CREATE TABLE ticket_watcher (
 CREATE TABLE ticket_index (
     ticket_id BIGINT NOT NULL,
     queue_id INTEGER NOT NULL,
-    queue VARCHAR (70) NOT NULL,
+    queue VARCHAR (200) NOT NULL,
     group_id INTEGER NOT NULL,
-    s_lock VARCHAR (70) NOT NULL,
-    s_state VARCHAR (70) NOT NULL,
+    s_lock VARCHAR (200) NOT NULL,
+    s_state VARCHAR (200) NOT NULL,
     create_time_unix BIGINT NOT NULL,
     INDEX ticket_index_group_id (group_id),
     INDEX ticket_index_queue_id (queue_id),
@@ -581,12 +544,6 @@ CREATE TABLE article (
     a_body MEDIUMTEXT NOT NULL,
     incoming_time INTEGER NOT NULL,
     content_path VARCHAR (250) NULL,
-    a_freekey1 VARCHAR (250) NULL,
-    a_freetext1 VARCHAR (250) NULL,
-    a_freekey2 VARCHAR (250) NULL,
-    a_freetext2 VARCHAR (250) NULL,
-    a_freekey3 VARCHAR (250) NULL,
-    a_freetext3 VARCHAR (250) NULL,
     valid_id SMALLINT NOT NULL,
     create_time DATETIME NOT NULL,
     create_by INTEGER NOT NULL,
@@ -613,12 +570,6 @@ CREATE TABLE article_search (
     a_message_id TEXT NULL,
     a_body MEDIUMTEXT NOT NULL,
     incoming_time INTEGER NOT NULL,
-    a_freekey1 VARCHAR (250) NULL,
-    a_freetext1 VARCHAR (250) NULL,
-    a_freekey2 VARCHAR (250) NULL,
-    a_freetext2 VARCHAR (250) NULL,
-    a_freekey3 VARCHAR (250) NULL,
-    a_freetext3 VARCHAR (250) NULL,
     PRIMARY KEY(id),
     INDEX article_search_article_sender_type_id (article_sender_type_id),
     INDEX article_search_article_type_id (article_type_id),
@@ -932,6 +883,7 @@ CREATE TABLE mail_account (
     account_type VARCHAR (20) NOT NULL,
     queue_id INTEGER NOT NULL,
     trusted SMALLINT NOT NULL,
+    imap_folder VARCHAR (250) NULL,
     comments VARCHAR (250) NULL,
     valid_id SMALLINT NOT NULL,
     create_time DATETIME NOT NULL,
@@ -979,7 +931,8 @@ CREATE TABLE process_id (
     process_name VARCHAR (200) NOT NULL,
     process_id VARCHAR (200) NOT NULL,
     process_host VARCHAR (200) NOT NULL,
-    process_create INTEGER NOT NULL
+    process_create INTEGER NOT NULL,
+    process_change INTEGER NOT NULL
 );
 # ----------------------------------------------------------
 #  create table web_upload_cache
@@ -1113,7 +1066,7 @@ CREATE TABLE virtual_fs (
     create_time DATETIME NOT NULL,
     PRIMARY KEY(id),
     INDEX virtual_fs_backend (backend(60)),
-    INDEX virtual_fs_filename (filename(350))
+    INDEX virtual_fs_filename (filename(255))
 );
 # ----------------------------------------------------------
 #  create table virtual_fs_preferences
@@ -1134,7 +1087,7 @@ CREATE TABLE virtual_fs_db (
     content LONGBLOB NOT NULL,
     create_time DATETIME NOT NULL,
     PRIMARY KEY(id),
-    INDEX virtual_fs_db_filename (filename(350))
+    INDEX virtual_fs_db_filename (filename(255))
 );
 # ----------------------------------------------------------
 #  create table package_repository
@@ -1155,3 +1108,139 @@ CREATE TABLE package_repository (
     change_by INTEGER NOT NULL,
     PRIMARY KEY(id)
 );
+# ----------------------------------------------------------
+#  create table gi_webservice_config
+# ----------------------------------------------------------
+CREATE TABLE gi_webservice_config (
+    id INTEGER NOT NULL AUTO_INCREMENT,
+    name VARCHAR (200) NOT NULL,
+    config LONGBLOB NOT NULL,
+    config_md5 VARCHAR (32) NOT NULL,
+    valid_id SMALLINT NOT NULL,
+    create_time DATETIME NOT NULL,
+    create_by INTEGER NOT NULL,
+    change_time DATETIME NOT NULL,
+    change_by INTEGER NOT NULL,
+    PRIMARY KEY(id),
+    UNIQUE INDEX gi_webservice_config_config_md5 (config_md5),
+    UNIQUE INDEX gi_webservice_config_name (name)
+);
+# ----------------------------------------------------------
+#  create table gi_webservice_config_history
+# ----------------------------------------------------------
+CREATE TABLE gi_webservice_config_history (
+    id BIGINT NOT NULL AUTO_INCREMENT,
+    config_id INTEGER NOT NULL,
+    config LONGBLOB NOT NULL,
+    config_md5 VARCHAR (32) NOT NULL,
+    create_time DATETIME NOT NULL,
+    create_by INTEGER NOT NULL,
+    change_time DATETIME NOT NULL,
+    change_by INTEGER NOT NULL,
+    PRIMARY KEY(id),
+    UNIQUE INDEX gi_webservice_config_history_config_md5 (config_md5)
+);
+# ----------------------------------------------------------
+#  create table scheduler_task_list
+# ----------------------------------------------------------
+CREATE TABLE scheduler_task_list (
+    id BIGINT NOT NULL AUTO_INCREMENT,
+    task_data TEXT NOT NULL,
+    task_data_md5 VARCHAR (32) NOT NULL,
+    task_type VARCHAR (200) NOT NULL,
+    due_time DATETIME NOT NULL,
+    create_time DATETIME NOT NULL,
+    PRIMARY KEY(id),
+    UNIQUE INDEX scheduler_task_list_task_data_md5 (task_data_md5)
+);
+# ----------------------------------------------------------
+#  create table gi_debugger_entry
+# ----------------------------------------------------------
+CREATE TABLE gi_debugger_entry (
+    id BIGINT NOT NULL AUTO_INCREMENT,
+    communication_id VARCHAR (32) NOT NULL,
+    communication_type VARCHAR (50) NOT NULL,
+    remote_ip VARCHAR (50) NULL,
+    webservice_id INTEGER NOT NULL,
+    create_time DATETIME NOT NULL,
+    PRIMARY KEY(id),
+    UNIQUE INDEX gi_debugger_entry_communication_id (communication_id),
+    INDEX gi_debugger_entry_create_time (create_time)
+);
+# ----------------------------------------------------------
+#  create table gi_debugger_entry_content
+# ----------------------------------------------------------
+CREATE TABLE gi_debugger_entry_content (
+    id BIGINT NOT NULL AUTO_INCREMENT,
+    gi_debugger_entry_id BIGINT NOT NULL,
+    debug_level VARCHAR (50) NOT NULL,
+    subject VARCHAR (255) NOT NULL,
+    content LONGBLOB NULL,
+    create_time DATETIME NOT NULL,
+    PRIMARY KEY(id),
+    INDEX gi_debugger_entry_content_create_time (create_time),
+    INDEX gi_debugger_entry_content_debug_level (debug_level)
+);
+# ----------------------------------------------------------
+#  create table gi_object_lock_state
+# ----------------------------------------------------------
+CREATE TABLE gi_object_lock_state (
+    webservice_id INTEGER NOT NULL,
+    object_type VARCHAR (30) NOT NULL,
+    object_id BIGINT NOT NULL,
+    lock_state VARCHAR (30) NOT NULL,
+    lock_state_counter INTEGER NOT NULL,
+    create_time DATETIME NOT NULL,
+    change_time DATETIME NOT NULL,
+    UNIQUE INDEX gi_object_lock_state_list (webservice_id, object_type, object_id),
+    INDEX object_lock_state_list_state (webservice_id, object_type, object_id, lock_state)
+);
+# ----------------------------------------------------------
+#  create table smime_signer_cert_relations
+# ----------------------------------------------------------
+CREATE TABLE smime_signer_cert_relations (
+    id INTEGER NOT NULL AUTO_INCREMENT,
+    cert_hash VARCHAR (8) NOT NULL,
+    cert_fingerprint VARCHAR (59) NOT NULL,
+    ca_hash VARCHAR (8) NOT NULL,
+    ca_fingerprint VARCHAR (59) NOT NULL,
+    create_time DATETIME NOT NULL,
+    create_by INTEGER NOT NULL,
+    change_time DATETIME NOT NULL,
+    change_by INTEGER NOT NULL,
+    PRIMARY KEY(id)
+);
+# ----------------------------------------------------------
+#  create table dynamic_field_value
+# ----------------------------------------------------------
+CREATE TABLE dynamic_field_value (
+    id INTEGER NOT NULL AUTO_INCREMENT,
+    field_id INTEGER NOT NULL,
+    object_id BIGINT NOT NULL,
+    value_text TEXT NULL,
+    value_date DATETIME NULL,
+    value_int BIGINT NULL,
+    PRIMARY KEY(id),
+    INDEX index_field_values (field_id, object_id),
+    INDEX index_search_date (field_id, value_date),
+    INDEX index_search_int (field_id, value_int)
+);
+# ----------------------------------------------------------
+#  create table dynamic_field
+# ----------------------------------------------------------
+CREATE TABLE dynamic_field (
+    id INTEGER NOT NULL AUTO_INCREMENT,
+    name VARCHAR (200) NOT NULL,
+    label VARCHAR (200) NOT NULL,
+    field_order INTEGER NOT NULL,
+    field_type VARCHAR (200) NOT NULL,
+    object_type VARCHAR (200) NOT NULL,
+    config LONGBLOB NULL,
+    valid_id SMALLINT NOT NULL,
+    create_time DATETIME NOT NULL,
+    create_by INTEGER NOT NULL,
+    change_time DATETIME NOT NULL,
+    change_by INTEGER NOT NULL,
+    PRIMARY KEY(id),
+    UNIQUE INDEX dynamic_field_name (name)
+);
