@@ -1,8 +1,6 @@
 // --
 // Core.UI.Table.js - Table specific functions
-// Copyright (C) 2001-2011 OTRS AG, http://otrs.org/\n";
-// --
-// $Id: Core.UI.Table.js,v 1.5 2011/01/21 12:12:22 mn Exp $
+// Copyright (C) 2001-2012 OTRS AG, http://otrs.org/
 // --
 // This software comes with ABSOLUTELY NO WARRANTY. For details, see
 // the enclosed file COPYING for license information (AGPL). If you
@@ -37,7 +35,7 @@ Core.UI.Table = (function (TargetNS) {
     TargetNS.InitCSSPseudoClasses = function ($Context) {
         var SelectorCount = 0;
         if (typeof $Context === 'undefined' || (isJQueryObject($Context) && $Context.length)) {
-            // comma-seperated selectors have performance issues, so we add the different selectors after each other
+            // comma-separated selectors have performance issues, so we add the different selectors after each other
             $('tr.Even', $Context)
                 .add('tr.Last', $Context)
                 .add('th.Last', $Context)
@@ -47,8 +45,8 @@ Core.UI.Table = (function (TargetNS) {
                 .removeClass('Even Last');
 
             // nth-child selector has heavy performance problems on big tables or lists
-            // Because this CSS classes are only used on IE7, we skip this class for big tables and lists
-            SelectorCount = $('tr', $Context).length + $('li:not(.Header)', $Context).length
+            // Because these CSS classes are only used on IE7, we skip these for big tables and lists
+            SelectorCount = $('tr', $Context).length + $('li:not(.Header)', $Context).length;
             if (SelectorCount < 200) {
                 $('tr:nth-child(even)', $Context)
                 .add('li:not(.Header):nth-child(even)', $Context)
@@ -56,14 +54,57 @@ Core.UI.Table = (function (TargetNS) {
             }
 
             // comma-seperated selectors have performance issues, so we add the different selectors after each other
-            $('tr:last-child', $Context)
-                .add('th:last-child', $Context)
-                .add('td:last-child', $Context)
-                .add('li:last-child', $Context)
-                .addClass('Last');
+            $('tr:last-child', $Context).addClass('Last');
+            $('th:last-child', $Context).addClass('Last');
+            $('td:last-child', $Context).addClass('Last');
+            $('li:last-child', $Context).addClass('Last');
         }
     };
 
+    /**
+     * @function
+     * @description
+     *      This function re-calculates some css values for the fixed table headers.
+     *      The ControlRow can be of a different height in some cases. That needs adjustment of the following elements.
+     * @return nothing
+     */
+    TargetNS.InitFixedHeader = function () {
+        var $ControlRow,
+            ControlRowHeight = 0,
+            ControlRowLineHeight = 25,
+            FixedHeaderAdjustement = 0,
+            FixedHeaderTopPosition,
+            FixedHeaderContainerPadding;
+
+        // Only if a fixed table exists
+        if (!$('#FixedTable').length) {
+            return;
+        }
+
+        $ControlRow = $('.OverviewControl .ControlRow');
+
+        if (!$ControlRow.length) {
+            return;
+        }
+
+        ControlRowHeight = $ControlRow.height();
+        // Default CSS is defined for a ControlRow with one line
+        FixedHeaderAdjustement = ControlRowHeight - ControlRowLineHeight;
+
+        // Only continue if ControlRow has more than one line
+        if (FixedHeaderAdjustement <= 0) {
+            return;
+        }
+
+        // Adjust CSS
+        FixedHeaderContainerPadding = parseInt($('.Overview.FixedHeader').css('padding-top'), 10);
+        FixedHeaderTopPosition = parseInt($('.Overview.FixedHeader thead tr').css('top'), 10);
+
+        $('.Overview.FixedHeader').css('padding-top', (FixedHeaderContainerPadding + FixedHeaderAdjustement - 0) + 'px');
+        $('.Overview.FixedHeader thead tr').css('top', (FixedHeaderTopPosition + FixedHeaderAdjustement - 0) + 'px');
+
+    };
+
     /**
      * @function
      * @description
@@ -97,18 +138,23 @@ Core.UI.Table = (function (TargetNS) {
                  * @description Ckeck if a text exist inside an element
                  */
                 function CheckText($Element, FilterText) {
-                    if ($Element.text().toLowerCase().indexOf(FilterText) > -1){
+                    var Text;
+
+                    Text = $Element.text();
+                    if (Text && Text.toLowerCase().indexOf(FilterText) > -1){
                         return true;
                     }
 
                     if ($Element.is('li, td')) {
-                        if ($Element.attr('title').toLowerCase().indexOf(FilterText) > -1) {
+                        Text = $Element.attr('title');
+                        if (Text && Text.toLowerCase().indexOf(FilterText) > -1) {
                             return true;
                         }
                     }
                     else {
                         $Element.find('td').each(function () {
-                            if ($(this).attr('title').toLowerCase().indexOf(FilterText) > -1) {
+                            Text = $(this).attr('title');
+                            if (Text && Text.toLowerCase().indexOf(FilterText) > -1) {
                                 return true;
                             }
                         });
